# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/controls.py
#
# \brief All individual controls
# 
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
# 
# \par License:
# GPL version 2.0

import re
import sys
import wx
import wx.lib.mixins.listctrl

import config
import images
import helper
import smm


## Generic functions to save and restore the state of a panel
class GeneralPanelConfig(object):
    
    ## Section in the configuration that contains the current look
    config_section = None

    ## Dictionary with all elements to save/restore
    config_items = {
        # attr    type    
        'list': 'ListCtrl',
        }

    ## Save the current configuration before the panel is closed
    #
    #  The parent class should bind this event handler
    #  \see restoreConfig() and saveConfig()
    def OnClose(self, event):
        self.saveConfig()
        event.Skip()

    ## Restore saved settings
    #
    #  The settings are not restored if \ref config_section is not setted.
    #  \see config_section, saveConfig()
    def restoreConfig(self):
        if not self.config_section:
            return
        for name in self.config_items.iterkeys():
            attr = getattr(self, name)
            kind = self.config_items[name]
            if kind == 'ListCtrl':
                self.restoreListCtrl(attr, self.config_section, name)
            elif kind == 'SplitterWindow':
                attr.SetSashPosition(
                    config.get_item('%s/sash_position' % self.config_section)
                    )
            else:
                raise ValueError, 'Unknown type for attribute %s' % name

    ## Save current settings
    #
    #  The settings are not restored if \ref config_section is not setted.
    #  \see config_section, restoreConfig()
    def saveConfig(self):
        if not self.config_section:
            return
        for name in self.config_items.iterkeys():
            attr = getattr(self, name)
            kind = self.config_items[name]
            if kind == 'ListCtrl':
                self.saveListCtrl(attr, self.config_section, name)
            elif kind == 'SplitterWindow':
                config.set_item(
                    '%s/sash_position' % self.config_section,
                    attr.GetSashPosition()
                    )
            else:
                raise ValueError, 'Unknown type for attribute %s' % name

    ## Restore saved settings
    #
    #  All exceptions raised during setting the colum widths will
    #  droped
    #
    #  \see saveListCtrl()
    def restoreListCtrl(self, ctrl, section, prefix):
        key = '%s/%s-column_widths' % (section, prefix)
        widths = config.get_item(key)
        if widths:
            for pos in range(ctrl.GetColumnCount()):
                try:
                    ctrl.SetColumnWidth(pos, int(widths[pos]))
                except:
                    pass

    ## Save current settings
    #
    #  \see restoreListCtrl()
    def saveListCtrl(self, ctrl, section, prefix):
        key = '%s/%s-column_widths' % (section, prefix)
        widths = []
        for col in range(ctrl.GetColumnCount()):
            widths.append('%d' % ctrl.GetColumnWidth(col))
        config.set_item(key, widths)


## Base class of a %panel with a sortable list inside
class SortedListPanel(wx.Panel, GeneralPanelConfig):

    ## Section in the configuration that contains the current look
    config_section = None

    ## Style of the ListCtrl
    listStyle = wx.LC_REPORT|wx.SUNKEN_BORDER

    ## Reference to the used SortedListCtrl
    list = None

    ## Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base classes
        wx.Panel.__init__(self, *args, **kwargs)
        GeneralPanelConfig.__init__(self)

        # create list
        self.createList()
        self.populateList()

        # restore saved settings
        self.restoreConfig()

        # position ListCtrl
        box = wx.BoxSizer(wx.HORIZONTAL)
        box.Add(self.list, 1, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(box)

    ## Create the header of a list
    def createListHeader(self):
        raise NotImplementedError, \
            'Implement this funtion in the derivated class'

    ## Create list, add header and enable sorting
    #
    #  \see populateList(), createListHeader() and
    #       SortedListCtrl.enableColumnSorter()
    def createList(self):
        # create ListCtrl first
        self.list = SortedListCtrl(
            self, 
            -1,
            style=self.listStyle
            )

        # create head line
        self.createListHeader()

        # enable the column sorter initially
        self.list.enableColumnSorter()

    ## Fill a already created list.
    #
    #  Derivated classes can use this function and add individual code
    #
    #  \see createList()
    def populateList(self):
        # delete existing entries and initialise variables
        self.list.DeleteAllItems()

    ## Remove a listed entry
    #
    #  \param self        The object instance itself
    #  \param element_id  ID of the entry
    def removeEntry(self, element_id):
        item = self.list.FindItemData(-1, element_id)
        if item == -1:
            return
        self.list.DeleteItem(item)


## Sortable ListCtrl
#
#  Use this control only for combined lists (e.g. click to one list item
#  causes an action of the other list) in a widget.
#
#  Please use SortedListPanel for single lists!
#
#  \note This implementation uses code of the ColumnSorterMixin. Some
#        pieces of code are copiied in.
class SortedListCtrl(wx.ListCtrl, wx.lib.mixins.listctrl.ColumnSorterMixin):

    ## \var itemDataMap
    #  Mapping between the index and the row values
    #
    #  This structure is used to sort the columns

    ## \var img_list
    #  List of images used by this list control

    ## \var upArrow
    #  Small image with an arrow point up

    ## \var downArrow
    #  Small image with an arrow point down

    ## \var columnSorterEnabled
    #  Flag to control the state of column sorter
    #  \see enableColumnSorter() and disableColumnSorter()

    ## \var currentItem
    #  Index of the currently selected item
    #
    #  \c None means that no item has been selected 
    #
    #  \see _OnItemSelected()

    ## Constructor, that initialise the wx.ListCtrl only.
    #
    #  We don't use the constructor of the class ColumnSorterMixin. The 
    #  functionality is moved to \ref enableColumnSorter().
    def __init__(self, parent, *args, **kwargs):
        # add default list style
        if not kwargs.has_key('style'):
            kwargs['style'] = wx.LC_REPORT|wx.SUNKEN_BORDER

        # initialise base class
        wx.ListCtrl.__init__(self, parent, *args, **kwargs)

        # initialise on variables
        self.columnSorterEnabled = False
        self.currentItem = None
        self.itemDataMap = {}
        self._col = -1

        # add small up / down arrows
        self.img_list = wx.ImageList(16, 16)
        self.upArrow = self.img_list.Add(images.getSmallUpArrowBitmap())
        self.downArrow = self.img_list.Add(images.getSmallDnArrowBitmap())
        self.SetImageList(self.img_list, wx.IMAGE_LIST_SMALL)

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self._OnItemSelected)

    ## Enables the column sorter
    #
    #  Calling this function is mandatory cause the ColumnSorterMixin
    #  will NOT be initialised during the constructor runs!
    #
    #  \see disableColumnSorter() and columnSorterEnabled
    def enableColumnSorter(self):
        self.SetColumnCount(self.GetColumnCount())
        self.Bind(wx.EVT_LIST_COL_CLICK, self._OnColClick)
        self.columnSorterEnabled = True

    ## Delete all items and clean \ref currentItem too
    def DeleteAllItems(self):
        wx.ListCtrl.DeleteAllItems(self)
        self.currentItem = None
        self.itemDataMap = {}

    ## Disables the column sorter
    #
    #  Disconnects the event handler from the event only.
    #
    #  \see enableColumnSorter() and columnSorterEnabled
    def disableColumnSorter(self):
        if self.columnSorterEnabled:
            self.Unbind(wx.EVT_LIST_COL_CLICK, self._OnColClick)
            self.ClearColumnImage(self._col)
            self.columnSorterEnabled = False

    ## Returns the list to be sorted
    #
    #  This function is needed by the ColumnSorterMixin
    def GetListCtrl(self):
        return self

    ## Returns a column sorter, that support unicode
    #
    #  \see unicodeColumnSorter()
    def GetColumnSorter(self):
        return self.unicodeColumnSorter

    ## Returns a tuple of %images that shows the sorting order in
    #   the column header
    def GetSortImages(self):
        return (self.upArrow, self.downArrow)

    ## Sort the ListCtrl again using the current settings
    def ReSort(self):
        self.SortListItems(col=self._col)

    ## Simple sort of two keys 
    #
    #  The sorter bases at the original code. The locale stuff has been
    #  removed only.
    def unicodeColumnSorter(self, key1, key2):
        col = self._col
        ascending = self._colSortFlag[col]
        item1 = self.itemDataMap[key1][col]
        item2 = self.itemDataMap[key2][col]

        cmpVal = cmp(item1, item2)

        # If the items are equal then pick something else to make the sort
        # value unique
        if cmpVal == 0:
            cmpVal = cmp(*self.GetSecondarySortValues(col, key1, key2))

        if ascending:
            return cmpVal
        else:
            return -cmpVal

    ## Event handler of clicks at the column headers
    def _OnColClick(self, evt):
        oldCol = self._col
        self._col = col = evt.GetColumn()
        self._colSortFlag[col] = not self._colSortFlag[col]
        self.SortItems(self.GetColumnSorter())
        self._updateImages(oldCol)
        evt.Skip()

    ## Store the index of the selected item in \ref currentItem
    def _OnItemSelected(self, event):
        self.currentItem = event.m_itemIndex
        event.Skip()

    ## Updates the sort images in the column headers
    def _updateImages(self, oldCol):
        sortImages = self.GetSortImages()
        if self._col != -1 and sortImages[0] != -1:
            img = sortImages[self._colSortFlag[self._col]]
            if oldCol != -1:
                self.ClearColumnImage(oldCol)
            self.SetColumnImage(self._col, img)


## Base class for a simple list to fill with elements of smm.objects
class BaseListSmall(object):

    ## Reference to a SortedListCtrl
    control = None

    ## Create the list header and enable the column sorter.
    #
    #  \param self     The object instance itself
    #  \param control  Reference to a already created SortedListCtrl
    def __init__(self, control):
        self.control = control
        self.createHeader()
        self.control.enableColumnSorter()

    ## Add a single objects object to the list
    #
    #  \param self    The object instance itself
    #  \param element A single object
    def add(self, element):
        raise NotImplementedError, \
            'Implement this funtion in the derivated class'

    ## Clean the list.
    def clean(self):
        self.control.DeleteAllItems()

    ## Creates a header only list
    def createHeader(self):
        raise NotImplementedError, \
            'Implement this funtion in the derivated class'

    ## Creates a list of smm.objects.Position objects 
    #
    #  \param self          The object instance itself
    #  \param element_list List of smm.objects.Position objects
    def populate(self, element_list):
        # delete existing entries
        self.control.DeleteAllItems()

        # build new list
        for element in element_list:
            self.add(element)

        # sort list by share name initially
        self.control.SortListItems(col=0)


## Class to simpilfy creating and controlling of Position lists.
#
#  Wrapper class to consolidates code to maintain Position lists.
class PositionListSmall(BaseListSmall):
    
    ## Creates a header only list
    def createHeader(self):
        self.control.InsertColumn(0, _(u'Wertpapier'))
        self.control.InsertColumn(1, _(u'Erffnet am'))
        self.control.InsertColumn(2, _(u'Geschlossen am'))
        self.control.InsertColumn(3, _(u'Status'))

    ## Add a single smm.objects.Position object to the list
    #
    #  \param self      The object instance itself
    #  \param position  A single smm.objects.Position
    def add(self, position):
        share_show = position.share.name
        share_sort = share_show.lower()
        first_show = u''
        first_sort = 0
        last_show  = u''
        last_sort  = 0

        # get date of first and last transaction
        count = position.transactions.count()
        if count >= 1:
            ft = position.transactions.order_by(smm.Transaction.date).first()
            first_show = helper.formatDate(ft.date)
            first_sort = str(ft.date)
        if count >= 2 and not position.open:
            lt = position.transactions.order_by(smm.Transaction.date).last()
            last_show = helper.formatDate(lt.date)
            last_sort = str(lt.date)

        # get status of the position
        if position.open:
            status_show = _(u'Offen')
        else:
            status_show = _(u'Geschlossen')
        status_sort = status_show.lower()

        index = self.control.InsertStringItem(sys.maxint, share_show)
        self.control.SetStringItem(index, 1, first_show)
        self.control.SetStringItem(index, 2, last_show)
        self.control.SetStringItem(index, 3, status_show)

        # store data into itemDataMap
        self.control.itemDataMap[position.id] = (
            share_sort,
            first_sort,
            last_sort,
            status_sort,
            )
        self.control.SetItemData(index, position.id)


## Class to simpilfy creating and controlling of Transaction lists.
#
#  Wrapper class to consolidates code to maintain Transaction lists.
class TransactionListSmall(BaseListSmall):
    
    ## Creates a header only list
    def createHeader(self):
        self.control.InsertColumn(0, _(u'Datum'))
        self.control.InsertColumn(1, _(u'Anzahl'), wx.LIST_FORMAT_RIGHT)
        self.control.InsertColumn(2, _(u'Brse'))
        self.control.InsertColumn(3, _(u'Typ'))
        self.control.InsertColumn(4, _(u'Stckpreis'), wx.LIST_FORMAT_RIGHT)
        self.control.InsertColumn(5, _(u'Volumen'), wx.LIST_FORMAT_RIGHT)
        self.control.InsertColumn(6, _(u'Gebhr'), wx.LIST_FORMAT_RIGHT)
        self.control.InsertColumn(7, _(u'Haltedauer'))

    ## Add a single smm.objects.Transaction object to the list
    #
    #  \param self         The object instance itself
    #  \param transaction  A single \ref smm.objects.Transaction "transaction"
    def add(self, transaction):
        count_show    = helper.formatDecimal(transaction.count, 3)
        count_sort    = transaction.count
        date_show     = helper.formatDate(transaction.date)
        date_sort     = str(transaction.date)
        exchange_show = helper.formatExchange(transaction.exchange)
        exchange_sort = exchange_show.lower()
        fee_show      = helper.formatMoney(
            transaction.fee,
            transaction.position.currency
            )
        fee_sort      = transaction.fee
        kind_show     = helper.formatTransactionKind(transaction)
        kind_sort     = kind_show.lower()
        value_show    = helper.formatMoney(
            transaction.value,
            transaction.position.currency
            )
        value_sort    = transaction.value
        volume_sort   = transaction.getVolume(withFee=False)
        volume_show   = helper.formatMoney(
            volume_sort,
            transaction.position.currency
            )
        if transaction.kind in ['buy', 'split']:
            holding_period_sort = transaction.getHoldingPeriod()
            holding_period_show = helper.formatDuration(holding_period_sort)
        else:
            holding_period_sort = -1
            holding_period_show = u''

        index = self.control.InsertStringItem(sys.maxint, date_show)
        self.control.SetStringItem(index, 1, count_show)
        self.control.SetStringItem(index, 2, exchange_show)
        self.control.SetStringItem(index, 3, kind_show)
        self.control.SetStringItem(index, 4, value_show)
        self.control.SetStringItem(index, 5, volume_show)
        self.control.SetStringItem(index, 6, fee_show)
        self.control.SetStringItem(index, 7, holding_period_show)

        # store data into itemDataMap
        self.control.itemDataMap[transaction.id] = (
            date_sort,
            count_sort,
            exchange_sort,
            kind_sort,
            value_sort,
            fee_sort,
            volume_sort,
            holding_period_sort
            )
        self.control.SetItemData(index, transaction.id)


## A class derivated from the wx.Choice widget to filling with a
#  set of objects.
#
#  The class extend the original handling of the wx.Choice widget by
#  implementing own functions like \ref Select() that handles
#  objects instead of simple strings. 
#
#  This changed behaviour simplify the usage with a larger list of
#  objects. Because is't possible to set and get the objects directly
#  instead of conversion to a string and setting / getting this string.
#
#  The class converts the objects internaly into strings using \ref formatFunc
#  and adds the strings to the choice widget.
class BaseChoice(wx.Choice):

    ## Flag to add an empty entry to of the widget during initialisation
    #
    #  \see addEmptyChoice()
    addEmpty = False

    ## \var idx_obj
    #  Dictionary to lookup relations between position and object
    #
    #  The key is the position in the \c wx.Choice widget and the value is
    #  the presented object like a share.
    #
    #  \see \ref helper.ReverseDict

    ## Constructor
    def __init__(self, *args, **kwargs):
        wx.Choice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # generate a list of (formatted object, object)
        if hasattr(self, 'getFunc') and hasattr(self, 'formatFunc'):
            elements = []
            getter = getattr(self, 'getFunc')
            formatter = getattr(self, 'formatFunc')
            for obj in getter():
                elements.append((formatter(obj), obj))

            # sort list
            elements.sort(cmp=lambda x,y: cmp(x[0].lower(), y[0].lower()))

            # fill reverse dictionary as well as the Choise widget
            self._addElements(elements)

        # add empty element first
        if self.addEmpty:
            self.addEmptyChoice()

    ## Add an empty choice to the widget
    #
    #  An empty choice refers to \c None.
    #
    #  \see addEmpty, _addElements()
    def addEmptyChoice(self):
        if not self.idx_obj.reverse.has_key(None):
            self._addElements([(u'', None)])

    ## Add the given elements to the Choice box
    #
    #  \param self     The object instance itself
    #  \param elements List of tuples with text and object
    def _addElements(self, elements):
        for formatted, obj in elements:
            self.Append(formatted)
            self.idx_obj[len(self.idx_obj)] = obj

    ## Returns the selected object or \c None
    def GetSelection(self):
        pos = wx.Choice.GetSelection(self)
        if self.idx_obj.has_key(pos):
            return self.idx_obj[pos]
        return None

    ## Select an other object
    def SetSelection(self, obj):
        if self.idx_obj.reverse.has_key(obj):
            pos = self.idx_obj.reverse[obj]
            wx.Choice.SetSelection(self, pos)

    ## Select the first (non-empty) element
    #
    #  \see addEmpty
    def SelectFirst(self):
        if self.addEmpty:
            pos = 1
        else:
            pos = 0
        wx.Choice.SetSelection(self, pos)

    ## Selecting a element specified by the given object
    #
    #  \param self The object instance itself
    #  \param obj  Object to select
    def Select(self, obj):
        if self.idx_obj.reverse.has_key(obj):
            pos = self.idx_obj.reverse[obj]
            wx.Choice.Select(self, pos)


## Choice widget to select a \ref smm.objects.Share "share".
class ShareChoice(BaseChoice):

    ## Function to format a single object
    def formatFunc(self, obj):
        return helper.formatShare(obj)

    ## Function to query all objects of unique type
    def getFunc(self):
        return smm.Share.get()


## Choice widget to select a \ref smm.objects.Currency "currency".
class CurrencyChoice(BaseChoice):

    ## Function to format a single object
    def formatFunc(self, obj):
        return helper.formatCurrencyShort(obj)

    ## Function to query all objects of unique type
    def getFunc(self):
        return smm.Currency.get()


## Choice widget to select encoding of a file
#
#  The entries are set directly without usage of \ref getFunc and \ref
#  formatFunc.
class EncodingChoice(BaseChoice):

    ## Constructor
    def __init__(self, *args, **kwargs):
        BaseChoice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # add elements
        elements = [
            (_(u'ASCII'),                       'ascii'),
            (_(u'Unicode (UTF-8)'),             'utf-8'),
            (_(u'Westeuropisch (ISO 8859-1)'), 'latin1'),
            ]

        self._addElements(elements)


## Choice widget to select a \ref smm.objects.Exchange "exchange".
class ExchangeChoice(BaseChoice):
    addEmpty = True

    ## Function to format a single object
    def formatFunc(self, obj):
        return helper.formatExchange(obj)

    ## Function to query all objects of unique type
    def getFunc(self):
        return smm.Exchange.get()


## Choice widget to select verbosity (count) of log messages
#
#  The entries are set directly without usage of \ref getFunc and \ref
#  formatFunc.
class LogLevelChoice(BaseChoice):

    ## Constructor
    def __init__(self, *args, **kwargs):
        BaseChoice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # add elements
        elements = [
            (_(u'Hoch'),    'debug'),
            (_(u'Mittel'),  'info'),
            (_(u'Niedrig'), 'warning'),
            ]

        self._addElements(elements)


## Choice widget to select the transaction type
#
#  The entries are set directly without usage of \ref getFunc and \ref
#  formatFunc.
#
#  \see smm.objects.Transaction.kind
class TransactionKindChoice(BaseChoice):

    ## Constructor
    def __init__(self, *args, **kwargs):
        BaseChoice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # add elements
        elements = [
            (_(u'Kaufen'),      'buy'),
            (_(u'Verkaufen'),   'sell'),
            (_(u'Dividende'),   'dividend'),
            (_(u'Aktiensplit'), 'split'),
            ]

        self._addElements(elements)


## Choice widget to select the share type
#
#  The entries are set directly without usage of \ref getFunc and \ref
#  formatFunc.
#
#  \see smm.objects.Share.kind
class ShareKindChoice(BaseChoice):

    ## Constructor
    def __init__(self, *args, **kwargs):
        BaseChoice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # add elements
        elements = [
            (_(u'Aktie'),           'share'),
            (_(u'Index'),           'index'),
            (_(u'Fond'),            'fond'),
            (_(u'ETF'),             'etf'),
            (_(u'Optionsschein'),   'warrant'),
            (_(u'Zertifikat'),      'certificate'),
            (_(u'Anleihe'),         'bond'),
            ]

        self._addElements(elements)


## Choice widget to select the share source
#
#  The entries are set directly without usage of \ref getFunc and \ref
#  formatFunc.
#
#  \see smm.objects.Share.source
class ShareSourceChoice(BaseChoice):

    ## Constructor
    def __init__(self, *args, **kwargs):
        BaseChoice.__init__(self, *args, **kwargs)
        self.idx_obj = helper.ReverseDict()

        # add elements
        elements = [(x[1], x[0]) for x in smm.source2name.items()]
        self._addElements([(u'Automatische Auswahl', None)])
        self._addElements(elements)


## Base class for all validators
class BaseValidator(wx.PyValidator):

    ## Constructor
    def __init__(self):
        wx.PyValidator.__init__(self)

    ## Standard cloner
    #
    ## \note Every validator must implement the Clone() method.
    def Clone(self):
        raise NotImplementedError, \
            'Implement this funtion in the derivated class'

    ## Validate the contents of the given control.
    def Validate(self, win):
        return True

    ## Transfer data from validator to window.
    #
    #   The default implementation returns False, indicating that an error
    #   occurred.  We simply return True, as we don't do any data transfer.
    def TransferToWindow(self):
        return True 

    ## Transfer data from window to validator.
    #
    #  The default implementation returns False, indicating that an error
    #   occurred.  We simply return True, as we don't do any data transfer.
    def TransferFromWindow(self):
        return True


## Validator to check for valid decimal
#
#  The key events will be checked for valid characters as well as the
#  whole input is checked for valid decimal using a regular expression.
class DecimalValidator(BaseValidator):

    ## \var empty
    #  Allow empty widget (\c True) or not (\c False)

    ## Set of valid characters
    validCharacters = '1234567890,'

    ## Compiled regular expression to match decimal values
    #
    #  A comma will be used as decimal point.
    RE_DecimalPattern = re.compile('\d+,??\d*$')

    ## Constructor
    #
    #  \param self   The object instance itself
    #  \param empty  Check for empty widgets (see \ref empty)
    def __init__(self, empty=True):
        BaseValidator.__init__(self)
        self.empty = empty
        self.Bind(wx.EVT_CHAR, self.OnChar)

    ## Standard cloner
    #
    ## \note Every validator must implement the Clone() method.
    def Clone(self):
        return DecimalValidator(self.empty)

    ## Validate the contents of the given text control.
    def Validate(self, win):
        textCtrl = self.GetWindow()
        text = textCtrl.GetValue()

        if text and not self.RE_DecimalPattern.match(text):
            msg = _(u'Dieses Eingabefeld soll eine Zahl enthalten.')
        elif  not self.empty and not text:
            msg = _(u'Dieses Eingabefeld darf nicht leer sein.')
        else:
            msg = None

        if msg:
            textCtrl.SetBackgroundColour(u'pink')
            textCtrl.SetFocus()
            textCtrl.Refresh()
            wx.MessageBox(msg, _(u"Fehlerhafte Eingabe"))
            return False

        textCtrl.SetBackgroundColour(
            wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
        textCtrl.Refresh()
        return True

    ## Check key events for valid characters
    #
    #  \see \ref validCharacters
    def OnChar(self, event):
        key = event.GetKeyCode()

        if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
            event.Skip()
            return

        if chr(key) in self.validCharacters:
            event.Skip()
            return

        if not wx.Validator_IsSilent():
            wx.Bell()

        # Returning without calling even.Skip eats the event before it
        # gets to the text control
        return


## Force filled TextCtrl widgets
class NotEmptyValidator(BaseValidator):

    ## Standard cloner
    #
    ## \note Every validator must implement the Clone() method.
    def Clone(self):
        return NotEmptyValidator()

    ## Validate the contents of the given text control.
    def Validate(self, win):
        textCtrl = self.GetWindow()
        text = textCtrl.GetValue()

        if not len(text):
            textCtrl.SetBackgroundColour("pink")
            textCtrl.SetFocus()
            textCtrl.Refresh()
            wx.MessageBox(
                _(u'Dieses Eingabefeld darf nicht leer sein.'),
                _(u'Fehlerhafte Eingabe')
                )
            return False

        textCtrl.SetBackgroundColour(
            wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
        textCtrl.Refresh()
        return True


## Force filled Choice widgets
class NotEmptyChoiceValidator(BaseValidator):

    ## Standard cloner
    #
    ## \note Every validator must implement the Clone() method.
    def Clone(self):
        return NotEmptyChoiceValidator()

    ## Validate the contents of the given text control.
    def Validate(self, win):
        ctrl = self.GetWindow()
        obj = ctrl.GetSelection()

        if not obj:
            ctrl.SetBackgroundColour("pink")
            ctrl.SetFocus()
            ctrl.Refresh()
            wx.MessageBox(
                _(u'Dieses Eingabefeld darf nicht leer sein.'),
                _(u'Fehlerhafte Eingabe')
                )
            return False

        ctrl.SetBackgroundColour(
            wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
        ctrl.Refresh()
        return True


## A %panel to show all information about a \ref smm.objects.Share "share"
class ShareDetailsPanel(wx.Panel):

    ## Flags used with all label widgets (wx.StaticText)
    flags_label = wx.ALL|wx.ALIGN_CENTER_VERTICAL

    ## Flags used for all input widgets
    flags_input = wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL

    ## Create the %panel
    def __init__(self, *args, **kwds):
        wx.Panel.__init__(self, *args, **kwds)

        staticbox   = wx.StaticBox(self,  -1, _(u"Informationen zum Wertpapier"))
        lb_name     = wx.StaticText(self, -1, _(u"Name:"))
        lb_Curreny  = wx.StaticText(self, -1, _(u"Whrung:"))
        lb_Exchange = wx.StaticText(self, -1, _(u"Handelsplatz:"))
        lb_WKN      = wx.StaticText(self, -1, _(u"WKN:"))
        lb_Symbol   = wx.StaticText(self, -1, _(u"Symbol:"))
        lb_ISIN     = wx.StaticText(self, -1, _(u"ISIN:"))
        lb_Kind     = wx.StaticText(self, -1, _(u"Typ:"))
        lb_Source   = wx.StaticText(self, -1, _(u"Datenquelle:"))
        lb_Notes    = wx.StaticText(self, -1, _(u"Bemerkungen:"))
        lb_Watch    = wx.StaticText(self, -1, _(u"Beobachten:"))

        self.cc_Currency = CurrencyChoice(self, -1, choices=[])
        self.cc_Exchange = ExchangeChoice(self, -1, choices=[])
        self.cc_Kind     = ShareKindChoice(self, -1, choices=[])
        self.cb_Watch    = wx.CheckBox(self, -1, "")
        self.tc_ISIN     = wx.TextCtrl(self, -1, "")
        self.tc_Name     = wx.TextCtrl(self, -1, "")
        self.tc_Notes    = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE)
        self.tc_Source   = wx.TextCtrl(self, -1, "", style=wx.TE_READONLY)
        self.tc_Symbol   = wx.TextCtrl(self, -1, "")
        self.tc_WKN      = wx.TextCtrl(self, -1, "")

        self.cc_Exchange.SetMinSize((60, 27))

        sz_bag       = wx.GridBagSizer(hgap=5, vgap=5)
        sz_box       = wx.BoxSizer(wx.VERTICAL)
        sz_staticbox = wx.StaticBoxSizer(staticbox, wx.VERTICAL)

        sz_bag.Add(lb_name,     pos=(0, 0), flag=self.flags_label)
        sz_bag.Add(lb_Curreny,  pos=(1, 0), flag=self.flags_label)
        sz_bag.Add(lb_Exchange, pos=(1, 2), flag=self.flags_label)
        sz_bag.Add(lb_WKN,      pos=(2, 0), flag=self.flags_label)
        sz_bag.Add(lb_Symbol,   pos=(2, 2), flag=self.flags_label)
        sz_bag.Add(lb_ISIN,     pos=(3, 0), flag=self.flags_label)
        sz_bag.Add(lb_Watch,    pos=(3, 2), flag=self.flags_label)
        sz_bag.Add(lb_Kind,     pos=(4, 0), flag=self.flags_label)
        sz_bag.Add(lb_Source,   pos=(4, 2), flag=self.flags_label)
        sz_bag.Add(lb_Notes,    pos=(5, 0), flag=self.flags_label)

        sz_bag.Add(self.tc_Name,     pos=(0, 1), flag=self.flags_input,
                   span=(1, 3))
        sz_bag.Add(self.cc_Currency, pos=(1, 1), flag=self.flags_input)
        sz_bag.Add(self.cc_Exchange, pos=(1, 3), flag=self.flags_input)
        sz_bag.Add(self.tc_WKN,      pos=(2, 1), flag=self.flags_input)
        sz_bag.Add(self.tc_Symbol,   pos=(2, 3), flag=self.flags_input)
        sz_bag.Add(self.tc_ISIN,     pos=(3, 1), flag=self.flags_input)
        sz_bag.Add(self.cb_Watch,    pos=(3, 3), flag=self.flags_input)
        sz_bag.Add(self.cc_Kind,     pos=(4, 1), flag=self.flags_input)
        sz_bag.Add(self.tc_Source,   pos=(4, 3), flag=self.flags_input)
        sz_bag.Add(self.tc_Notes,    pos=(5, 1), flag=self.flags_input,
                   span=(1, 3))

        sz_staticbox.Add(sz_bag, 1, wx.ALL|wx.EXPAND, 5)
        sz_bag.AddGrowableCol(3)

        sz_box.Add(sz_staticbox, 1, wx.EXPAND, 0)

        self.SetSizer(sz_box)
        sz_box.Fit(self)
        self.Layout()

## A %panel show show all information of a \ref smm.objects.Position
#  "position".
class PositionDetailsPanel(wx.Panel):

    ## Flags used with all label widgets (wx.StaticText)
    flags_label = wx.ALL|wx.ALIGN_CENTER_VERTICAL

    ## Flags used for all input widgets
    flags_input = wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL

    ## Create the %panel
    def __init__(self, *args, **kwds):
        wx.Panel.__init__(self, *args, **kwds)

        staticbox   = wx.StaticBox(self,  -1, _(u"Informationen zur Position"))
        lb_Currency = wx.StaticText(self, -1, _(u"Whrung:"))
        lb_Notes    = wx.StaticText(self, -1, _(u"Bemerkungen:"))
        lb_Share    = wx.StaticText(self, -1, _(u"Wertpapier:"))

        self.cc_Currency = CurrencyChoice(self, -1, choices=[])
        self.cc_Share    = ShareChoice(self, -1, choices=[])
        self.tc_Notes    = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE)

        self.cc_Share.SetMinSize((60, 27))

        sz_bag       = wx.GridBagSizer(hgap=5, vgap=5)
        sz_box       = wx.BoxSizer(wx.VERTICAL)
        sz_staticbox = wx.StaticBoxSizer(staticbox, wx.VERTICAL)

        sz_bag.Add(lb_Currency, pos=(0, 0), flag=self.flags_label)
        sz_bag.Add(lb_Share,    pos=(0, 2), flag=self.flags_label)
        sz_bag.Add(lb_Notes,    pos=(1, 0), flag=self.flags_label)

        sz_bag.Add(self.cc_Currency, pos=(0, 1), flag=self.flags_input)
        sz_bag.Add(self.cc_Share,    pos=(0, 3), flag=self.flags_input)
        sz_bag.Add(self.tc_Notes,    pos=(1, 1), flag=self.flags_input,
                   span=(1, 3))

        sz_staticbox.Add(sz_bag, 1, wx.ALL|wx.EXPAND, 5)
        sz_bag.AddGrowableCol(3)

        sz_box.Add(sz_staticbox, 1, wx.EXPAND, 0)

        self.SetSizer(sz_box)
        sz_box.Fit(self)
        self.Layout()


## A %panel show show all information of a \ref smm.objects.Watch
#  "watch list entry".
class WatchDetailsPanel(wx.Panel):

    ## Flags used with all label widgets (wx.StaticText)
    flags_label = wx.ALL|wx.ALIGN_CENTER_VERTICAL

    ## Flags used for all input widgets
    flags_input = wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL

    ## Create the %panel
    def __init__(self, *args, **kwds):
        wx.Panel.__init__(self, *args, **kwds)

        staticbox  = wx.StaticBox(self,  -1, _(u"Informationen zur Eintrag"))
        lb_created = wx.StaticText(self,  -1, _(u"Angelegt:"))
        lb_expire  = wx.StaticText(self,  -1, _(u"Verfllt:"))
        lb_Notes   = wx.StaticText(self, -1, _(u"Bemerkungen:"))
        lb_Share   = wx.StaticText(self, -1, _(u"Wertpapier:"))

        self.cc_Share   = ShareChoice(self, -1, choices=[])
        self.cb_expire  = wx.CheckBox(self, -1, "")
        self.dp_created = wx.DatePickerCtrl(self, -1)
        self.dp_expire  = wx.DatePickerCtrl(self, -1)
        self.tc_Notes   = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE)

        self.cc_Share.SetMinSize((60, 27))

        sz_bag       = wx.GridBagSizer(hgap=5, vgap=5)
        sz_box       = wx.BoxSizer(wx.VERTICAL)
        sz_staticbox = wx.StaticBoxSizer(staticbox, wx.VERTICAL)

        sz_bag.Add(lb_Share,        pos=(0, 0), flag=self.flags_label)
        sz_bag.Add(self.cc_Share,   pos=(0, 1), flag=self.flags_input,
                   span=(1, 4))
        sz_bag.Add(lb_created,      pos=(1, 0), flag=self.flags_label, border=0)
        sz_bag.Add(self.dp_created, pos=(1, 1), flag=self.flags_input)
        sz_bag.Add(lb_expire,       pos=(1, 2), flag=self.flags_label)
        sz_bag.Add(self.cb_expire,  pos=(1, 3), flag=self.flags_input)
        sz_bag.Add(self.dp_expire,  pos=(1, 4), flag=self.flags_input)
        sz_bag.Add(lb_Notes,        pos=(2, 0), flag=self.flags_label)
        sz_bag.Add(self.tc_Notes,   pos=(2, 1), flag=self.flags_input,
                   span=(1, 4))

        sz_staticbox.Add(sz_bag, 1, wx.ALL|wx.EXPAND, 5)
        sz_bag.AddGrowableCol(1)
        sz_bag.AddGrowableCol(4)

        sz_box.Add(sz_staticbox, 1, wx.EXPAND, 0)

        self.Bind(wx.EVT_CHECKBOX, self.OnCheckExpired, self.cb_expire)

        if self.cb_expire.IsChecked():
            self.dp_expire.Enable(True)
        else:
            self.dp_expire.Enable(False)

        self.SetSizer(sz_box)
        sz_box.Fit(self)
        self.Layout()


    ## Enable / disable the expire date depending on the check box
    #  state.
    def OnCheckExpired(self, event):
        self.dp_expire.Enable(event.IsChecked())

