# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/dialogs.py
#
# \brief All dialogs
#
# \namespace gui::dialogs
# Implementation of the logic behind the dialogs.
# 
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
# 
# \par Licence:
# GPL version 2.0

import datetime
import decimal
import logging
import os.path
import re
import StringIO
import sys
import wx
import wx.lib.pubsub as pubsub
from wx.lib.wordwrap import wordwrap

import controls
import helper
import log
import reports
import smm
import UIFetchQuotes
import UITrade
import UIShare
import UIHtml
import UIPortfolioReport
import UIPositionReport
import UIPreferences


## Dialog to fetch quotes
class DlgFetchQuotes(UIFetchQuotes.UIFetchQuotes):

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    def __init__(self, parent):
        # call parent constructor
        UIFetchQuotes.UIFetchQuotes.__init__(self, parent)

        # bind additional events
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBoxClicked, self.cb_OpenPosition)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBoxClicked, self.cb_WatchList)
        self.Bind(wx.EVT_BUTTON, self.OnFetchQuotes, self.bn_fetch)
        self.bn_fetch.SetDefault()

    ## Enable the fetch button depending if at least one list checked
    def OnCheckBoxClicked(self, event):
        if self.cb_OpenPosition.IsChecked() or self.cb_WatchList.IsChecked():
            self.bn_fetch.Enable(True)
        else:
            self.bn_fetch.Enable(False)

    ## Fetch quotes
    #
    #  \see smm.operations.fetchDailyQuotes()
    def OnFetchQuotes(self, event):
        # disable and reenable input widgets
        self.bn_fetch.Enable(False)
        self.tc_output.Enable(True)

        count_watch = smm.Watch.get()
        count_open = smm.Position.get(open=True)

        if not count_watch and not count_open:
            wx.MessageBox(
                _(u'Es werden keine Kursinformationen abgefragt, da weder offene Positionen noch Eintrge in der Beobachtungsliste existieren.'),
                _(u'Hinweis'),
                wx.OK | wx.ICON_INFORMATION,
                )
            if self.IsModal():
                self.EndModal(wx.ID_OK)
            else:
                return

        # clear old log entries
        log.flush()

        # fetch quotations
        smm.fetchDailyQuotes(
            self.cb_OpenPosition.IsChecked(),
            self.cb_WatchList.IsChecked()
            )

        # show log messages
        messages = log.getBufferAsString()
        self.tc_output.WriteText(messages)

        # scroll to last line
        # Doesn't work really, it scrolls to the second last line only!
        # \bug TextCtrl doesn't scroll down till the last line!
        pos = self.tc_output.XYToPosition(
            0,
            self.tc_output.GetNumberOfLines() -1
            )
        self.tc_output.SetInsertionPoint(pos)
        self.tc_output.ShowPosition(pos)

        # send quotation update message
        pubsub.Publisher().sendMessage('smm.quote', None)

        # enable OK button on success
        self.bn_ok.Enable(True)
        self.bn_ok.SetDefault()
        self.bn_cancel.Enable(False)


## Short main text of the about dialog
about_description = _(
u'''"Share and Money Management" ist ein Programm zum Verwalten eines Portfolie.

Fr Fehlerreports sowie Anregungen zu neuen Funktionen wenden Sie
sich bitte an mail@carstengrohmann.de.

Versionen der Komponenten:
Python: %(py_version)s
Beautiful Soup: %(bs_version)s
Storm:    %(storm_version)s
wxPython: %(wx_version)s''')


## Licence text
about_licence = _(
u'''Copyright (c) 2007-2009 Carsten Grohmann

Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation
verffentlicht, weitergeben und/oder modifizieren, gem Version 2 der Lizenz.

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen
von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.

Sie sollten ein Exemplar der GNU General Public License zusammen mit
diesem Programm erhalten haben. Falls nicht, siehe
<http://www.gnu.org/licenses/>. 

"Share and Money Management" hat folgende Software Dritter, die verschieden
lizenziert ist, integriert:
 - Beautiful Soup ("New-style BSD" Lizenz)
 - Storm  (GNU Lesser General Public License (LGPL) Version 2.1)
 - Mako template library (MIT-Lizenz)

Details zu den einzelnen Lizenzen befinden sich in den individuellen Softwarepaketen.''')

## Show the about dialog
#
#  \param parent         The parent object
#  \param smm_version    Version of the SMM Library
#  \param storm_version  Version of the Storm framework
def showAboutDialog(parent, smm_version, storm_version):

    import BeautifulSoup

    py_version = _(u"%d.%d.%d %s #%d" % sys.version_info)

    # First we create and fill the info object
    info = wx.AboutDialogInfo()
    info.Name = _(u"Share and Money Management")
    info.Version = smm_version
    info.Copyright = _(u'Copyright (c) 2007-2009 Carsten Grohmann')
    info.Description = wordwrap(
        about_description % {
            'bs_version':    BeautifulSoup.__version__,
            'py_version':    py_version,
            'storm_version': storm_version,
            'wx_version':    wx.version(),
            },
        350, wx.ClientDC(parent))

    info.WebSite = _(u'http://www.carstengrohmann.de/smm.html')

    #info.Developers = [_(u"Carsten Grohmann")]

    info.License = about_licence

    wx.AboutBox(info)


## Dialog to insert and change \ref smm.objects.Transaction "transactions"
class DlgTrade(UITrade.UITrade):

    ## Precompiled regular expression to replace comma by point
    #
    #  \note This is German specific!
    re_comma = re.compile(r',')

    ## \var position
    #
    #  \ref smm.objects.Position "Position" to add the current 
    #  \ref smm.objects.Transaction "Transaction".
    #
    #  \see self.handle_pos

    ## \var share
    #  \ref smm.objects.Share "Share" to trade with.

    ## \var handle_pos
    #
    #  Add the current \ref smm.objects.Transaction "Transaction" to an
    #  existing \ref smm.objects.Position "Position" if \c True.
    #
    #  \see self.position

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    #  \param kind   Kind of transaction, valid are \c sell, \c buy or
    #                \c None
    #  \param share  A \ref smm.objects.Share "Share" object
    #  \param position    A \ref smm.objects.Position "Position" object
    #
    #  Creating this dialog using the arguments \c share and \c position
    #  causes an \c ValueError exception.
    #
    #  \exception ValueError For unknown transaction kinds
    #  \exception ValueError If share and position is set
    def __init__(self, parent, kind=None, share=None, position=None):
        # call parent constructor
        UITrade.UITrade.__init__(self, parent)

        # initialise internal variables
        self.handle_pos = False
        self.position   = None
        self.share      = None

        if share and position:
            raise ValueError, 'Arguments share and position are given.'

        # set choice widget for transaction kind
        if kind in ['buy', 'sell']:
            self.cc_TransactionKind.Select(kind)
        else:
            self.cc_TransactionKind.Select('buy')

        # set dialog title and all share related widgets
        self.SetTitle(_(u'Wertpapier handeln'))

        if position:
            self.setPosition(position)
        else:
            self.setShare(share)
        
        # bind additional events
        self.Bind(wx.EVT_CHOICE,   self.OnChoiceCurrency, self.cc_Currency)

        # set Validator objects
        self.cc_Currency.SetValidator(controls.NotEmptyChoiceValidator())
        self.tc_Count.SetValidator(controls.DecimalValidator(empty=False))
        self.tc_Value.SetValidator(controls.DecimalValidator(empty=False))
        self.tc_Fee.SetValidator(controls.DecimalValidator())

    ## Set share related widgets
    #
    #  \param self   The object pointer
    #  \param share  A \ref smm.objects.Share "Share" object
    def setShare(self, share):
        self.share = share
        self.handle_pos = False
        self.cc_Share.Select(self.share)
        if self.share:
            self.cc_Exchange.Select(self.share.exchange)
            self.cc_Currency.Select(self.share.currency)
            self.OnChoiceCurrency(None)
            if self.share.watch and self.share.watch.notes:
                self.cb_TransferComment.Enable(True)
                self.cb_TransferComment.SetValue(True)
                self.cb_AppendComment.Enable(self.share.notes != None)
                self.cb_AppendComment.SetValue(self.share.notes != None)
            else:
                self.cb_TransferComment.Enable(False)
                self.cb_TransferComment.SetValue(False)
                self.cb_AppendComment.Enable(False)
                self.cb_AppendComment.SetValue(False)
        else:
            self.cc_Exchange.Select(None)
            self.cc_Currency.Select(None)
            self.cb_TransferComment.Enable(False)
            self.cb_TransferComment.SetValue(False)
            self.cb_AppendComment.Enable(False)
            self.cb_AppendComment.SetValue(False)

        # Enable widgets possibly disabled by setPosition()
        self.cc_Currency.Enable(True)
        self.cc_Share.Enable(True)


    ## Set position related widgets
    #
    #  \param self      The object pointer
    #  \param position  A \ref smm.objects.Share "Position" object
    def setPosition(self, position):
        self.position = position
        self.handle_pos = True

        self.cc_Currency.Select(self.position.currency)
        self.cc_Share.Select(self.position.share)
        self.OnChoiceCurrency(None)

        self.cb_TransferComment.Enable(False)
        self.cb_TransferComment.SetValue(False)
        self.cb_AppendComment.Enable(False)
        self.cb_AppendComment.SetValue(False)

        # Enable widgets possibly disabled by setPosition()
        self.cc_Currency.Enable(False)
        self.cc_Share.Enable(False)


    ## Update currency text %controls after the currency choice has changed
    def OnChoiceCurrency(self, event):
        cur = self.cc_Currency.GetSelection()
        short = helper.formatCurrencyShort(cur)
        self.tc_Currency1.SetValue(short)
        self.tc_Currency2.SetValue(short)

    ## Show the dialog modal and create a transaction using the input
    def ShowModal(self):
        res = UITrade.UITrade.ShowModal(self)
        if res != wx.ID_OK:
            return res
        self._process()
        return wx.ID_OK

    ##  Create a \ref smm.objects.Transaction "transaction" using all
    #   data
    def _process(self):

        # List of update events
        events = []

        # query input values
        count    = self.tc_Count.GetValue()
        date     = self.dp_Date.GetValue()
        exchange = self.cc_Exchange.GetSelection()
        fee      = self.tc_Fee.GetValue()
        kind     = self.cc_TransactionKind.GetSelection()
        value    = self.tc_Value.GetValue()

        # convert german decimal point "," into "." used by Python
        # internally
        count = self.re_comma.sub(u'.', count)
        value = self.re_comma.sub(u'.', value)

        # convert values into python objects
        count = decimal.Decimal(count)
        date  = helper.wxdate2pydate(date)
        if fee:
            fee = self.re_comma.sub(u'.', fee)
            fee = decimal.Decimal(fee)
        else:
            fee = None
        value = decimal.Decimal(value)

        # change the sign for "sell" transactions
        if kind == 'sell' and count > 0:
            count = count * decimal.Decimal('-1')

        if self.handle_pos:
            pos = self.position
        else:
            share = self.cc_Share.GetSelection()
            pos = smm.Position.get(share=share, open=True)
            if not pos:
                pos = smm.Position(
                    share=share,
                    currency=self.cc_Currency.GetSelection()
                    )
                pos.add2db(False)

        # add proper transaction
        func = getattr(
            pos,
            kind,
            )
        trans = func(count=count,
            date=date,
            exchange=exchange,
            fee=fee,
            value=value,
            )

        # close position on demand
        if self.cb_AutoClose.IsChecked() and \
           pos.getShareCount() == decimal.Decimal("0"):
            pos.open=False

        events += [
            ('smm.position.change', pos.id),
            ('smm.transaction.add', trans.id),
            ]


        # copy or move notes from a Watch object to the position object
        watch = pos.share.watch
        if watch:
            if watch.notes and self.cb_TransferComment.GetValue():
                if self.cb_AppendComment.GetValue():
                    text = _(
                        u'Kommentar des Beobachtungslisteneintrages:\n'
                        u'==========================================\n'
                        u'%s' % watch.notes)
                    if pos.notes:
                        pos.notes += u'\n\n'
                        pos.notes += text
                    else:
                        pos.notes = text
                else:
                    pos.notes = watch.notes


            # delete watch list entry finally
            events += [('smm.watch.remove', watch.id)]
            watch.remove()

        # commit database changes
        smm.database.store.commit()

        # send update events (not wx related)
        for ev in events: 
            pubsub.Publisher().sendMessage(*ev)

        # show position report if requested
        if self.cb_PositionReport.GetValue() and not pos.open:
            dlg = DlgShowPositionReport(self)
            dlg.ShowModal(pos)


## Dialog to add and change \ref smm.objects.Share "shares".
class DlgShare(UIShare.UIShare):

    ## Reference to a \ref smm.objects.Share "share".
    share = None

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    #  \param share  A \ref smm.objects.Share "Share" object
    def __init__(self, parent, share=None):
        # call parent constructor
        UIShare.UIShare.__init__(self, parent)

        if share:
            self._setShare(share)
        else:
            self.cc_Source.Select(None)

        # set Validator objects
        self.cc_Kind.SetValidator(controls.NotEmptyChoiceValidator())
        self.tc_Name.SetValidator(controls.NotEmptyChoiceValidator())


    ## Set the specified \ref smm.objects.Share.kind "share kind" and
    #  disable changing the kind
    #
    #  \param self  The object pointer
    #  \param kind  A \ref smm.objects.Share.kind "share kind"
    def setFixedShareKind(self, kind):
        self.cc_Kind.Select(kind)
        self.cc_Kind.Enable(False)

    ## Fill dialogs fields with share data
    #
    #  \param self   The object pointer
    #  \param share  A \ref smm.objects.Share "Share" object
    def _setShare(self, share):
        self.share = share
        self.SetTitle(_(u'Wertpapier ndern'))
        self.tc_Name.SetValue(self.share.name)
        if self.share.symbol:
            self.tc_Symbol.SetValue(self.share.symbol)
        if self.share.wkn:
            self.tc_WKN.SetValue(self.share.wkn)
        if self.share.isin:
            self.tc_ISIN.SetValue(self.share.isin)
        self.cc_Currency.Select(self.share.currency)
        self.cc_Exchange.Select(self.share.exchange)
        self.cc_Kind.Select(self.share.kind)
        self.cc_Source.Select(self.share.source)
        if self.share.notes:
            self.tc_Notes.SetValue(self.share.notes)


    ## Show the dialog modal and create a transaction using the input
    #
    #  The insert data will processed calling _process() on success.
    def ShowModal(self):
        res = UIShare.UIShare.ShowModal(self)
        if res != wx.ID_OK:
            return res
        self._process()
        return wx.ID_OK

    ##  Update existing \ref share or create a new one.
    #
    #   An existing share will be updated if \ref share is set.
    #   Otherwise a new share will be created using the dialog's data
    #   and stored in \ref share.
    def _process(self):
        # compose kwargs
        kwargs = {
            'name':     self.tc_Name.GetValue(),
            'symbol':   self.tc_Symbol.GetValue(),
            'wkn':      self.tc_WKN.GetValue(),
            'isin':     self.tc_ISIN.GetValue(),
            'currency': self.cc_Currency.GetSelection(),
            'exchange': self.cc_Exchange.GetSelection(),
            'kind':     self.cc_Kind.GetSelection(),
            'source':   self.cc_Source.GetSelection(),
            'notes':    self.tc_Notes.GetValue(),
            }

        # create or update a share
        if self.share:
            self.share.update(**kwargs)
            pubsub.Publisher().sendMessage(
                'smm.share.change',
                self.share.id
                )
        else:
            self.share = smm.Share(**kwargs)
            self.share.add2db()
            pubsub.Publisher().sendMessage(
                'smm.share.add',
                self.share.id
                )

## Generic dialog to show HTML and some buttons to print and save
class DlgHtml(UIHtml.UIHtml):

    ## Wildcards for selecting HTML files
    wildcard = _(u'HTML-Dokument (*.html)|*.html')

    ## \var page_content
    #  Store the page source
    #  \see setHtml()

    ## \var printing
    #  \brief Reference to the easy printing instance 

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    def __init__(self, parent):
        UIHtml.UIHtml.__init__(self, parent)
        self.page_content = None
        self.printing = wx.html.HtmlEasyPrinting(
            name=_(u'Drucken'), 
            parentWindow=None
            )

    ## Store \ref page_content in a file
    def save2file(self, filename):
        outfile = open(filename, 'w')
        outfile.write(self.page_content)
        outfile.close()

    ## Set the html source and display it
    #
    #  Use this function to store a copy of the page source for later
    #  processing e.g. saving.
    def setHtml(self, html):
        self.page_content = html
        self.hw_report.SetPage(self.page_content)

    ## Close only the modal dialog
    def OnClose(self, event):
        if self.IsModal():
            self.EndModal(wx.ID_OK)

    def OnPrintPreview(self, event):
        self.printing.PreviewText(self.page_content, basepath="")

    def OnPrint(self, event):
        self.printing.PrintText(self.page_content, basepath="")

    ## Save the content of the HTML widget
    def OnSave(self, event):
        dlg = wx.FileDialog(
            None,
            wildcard=self.wildcard,
            style=wx.SAVE
            )
        dlg.Centre(wx.BOTH)
        if dlg.ShowModal() == wx.ID_OK:
            # expand path
            path = os.path.abspath(os.path.expanduser(dlg.GetPath()))

            # check for extention
            if not path.endswith(u'.html') and not path.endswith(u'.htm'):
                path += u'.html'

            self.save2file(path)
        dlg.Destroy()


## Dialog to select start and end date for the portfolio report
#
#  \see gui.report.createPortfolioReport()
class DlgPortfolioReport(UIPortfolioReport.UIPortfolioReport):

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    def __init__(self, parent):
        # call parent constructor
        UIPortfolioReport.UIPortfolioReport.__init__(self, parent)

        # get date of first transaction
        first = smm.Transaction.get().first().date
        last = datetime.date.today()

        # fill years choice
        for year in range(first.year, last.year + 1):
            self.ccYear.Append(str(year))

        # set proper default values
        self.ccYear.SetStringSelection(str(last.year))
        self.dp_From.SetValue(wx.DateTimeFromDMY(1, 0, last.year))
        self.dp_To.SetValue(helper.pydate2wxdate(last))

        # Initial enable / disable of all widgets
        self.OnRadio(None)

        # set range for individual start and end date
        self.dp_From.SetRange(
            helper.pydate2wxdate(first),
            helper.pydate2wxdate(last),
            )
        self.dp_To.SetRange(
            helper.pydate2wxdate(first),
            helper.pydate2wxdate(last),
            )

        # bind additional events
        self.Bind(wx.EVT_RADIOBUTTON, self.OnRadio, self.rbn_CalendarYear)
        self.Bind(wx.EVT_RADIOBUTTON, self.OnRadio, self.rbn_FromTo)

    ## Enable / disable widgets depending on the active radio button
    def OnRadio(self, event):
        year = self.rbn_CalendarYear.GetValue()

        self.dp_From.Enable(not year)
        self.dp_To.Enable(not year)
        self.ccYear.Enable(year)

    ## Return the start date for a report
    #
    #  \see getEndDate()
    def getStartDate(self):
        date = datetime.date.today()
        if self.rbn_CalendarYear.GetValue():
            year = int(
                self.ccYear.GetStringSelection()
                )
            date = datetime.date(year, 1, 1)
        else:
            date = helper.wxdate2pydate(self.dp_From.GetValue())
        return date

    ## Return the end date for a report
    #
    #  \see getStartDate()
    def getEndDate(self):
        date = datetime.date.today()
        if self.rbn_CalendarYear.GetValue():
            year = int(
                self.ccYear.GetStringSelection()
                )
            date = datetime.date(year, 12, 31)
            if date > datetime.date.today():
                date = datetime.date.today()
        else:
            date = helper.wxdate2pydate(self.dp_To.GetValue())
        return date


## Dialog to select a position using different filter criteria
#
#  \see gui.report.createPositionReport()
class DlgPositionReport(UIPositionReport.UIPositionReport, controls.GeneralPanelConfig):

    ## \var cache
    #  Dictionary to cache search results

    config_section = 'DlgPositionReport'

    config_items = {
        'lc_Position': 'ListCtrl',
        }

    ## \var position_list
    #  \brief Instance to simplify controlling of the list of
    #         positions related to the current filter settings
    #  \see controls.PositionListSmall

    ## \var selection
    #  Stores the last selection to prevent unneeded position updates 

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    def __init__(self, parent):
        # call parent constructor
        UIPositionReport.UIPositionReport.__init__(self, parent)
        controls.GeneralPanelConfig.__init__(self)

        ## initialise instance variables
        self.selection = None
        self.cache = {}

        # create list
        self.position_list = controls.PositionListSmall(self.lc_Position)

        # fill position list initally
        self.OnRadio(None)

        # restore ListCtrl settings
        self.restoreConfig()

        # disable OK button till selecting first element
        self.bn_OK.Enable(False)

        # bind additional events
        self.Bind(wx.EVT_RADIOBOX, self.OnRadio, self.rbx_PositionOpen)
        self.Bind(
            wx.EVT_LIST_ITEM_SELECTED,
            self.OnActivateOK,
            self.lc_Position
            )
        self.Bind(
            wx.EVT_LIST_ITEM_ACTIVATED,
            self.OnShowReport,
            self.lc_Position
            )

    ## Activate OK button after first position is selected
    def OnActivateOK(self, event):
        if not self.bn_OK.IsEnabled():
            self.bn_OK.Enable(True)

    ## Update position list on changing the RadioBox selection
    #
    #  The search results returned by the database are cached in \ref
    #  cache.
    #
    #  \see cache and selection
    def OnRadio(self, event):
        selection = self.rbx_PositionOpen.GetSelection()
        # continue on changed selection only
        if self.selection == selection:
            return

        # Deactivate OK button after changing position list
        self.bn_OK.Enable(False)

        # show cached positions
        if self.cache.has_key(selection):
            self.position_list.populate(self.cache[selection])

        # closed positions
        if selection == 0:
            positions = smm.Position.get(open=False)
        # open positions
        elif selection == 1:
            positions = smm.Position.get(open=True)
        # open and closed positions
        else:
            positions = smm.Position.get()

        # Cache and display search
        self.cache[selection] = list(positions)
        self.position_list.populate(positions)

        # remember current selection
        self.selection = selection

    ## Returns the ID of the selected Position
    def GetSelection(self):
        return self.lc_Position.GetItemData(self.lc_Position.currentItem)

    ## Show the report for the selected position
    def OnShowReport(self, event):
        if self.IsModal():
            self.EndModal(wx.ID_OK)

    ## Show the dialog modal and save the config after afterwards
    def ShowModal(self):
        res = UIPositionReport.UIPositionReport.ShowModal(self)
        self.saveConfig()
        return res


## Dialog to change / set preferences
class DlgPreferences(UIPreferences.UIPreferences):

    ## \var db_file
    #  Database file to use

    ## \var log_enc
    #  Encoding of the log file

    ## \var log_level
    #  Verbosity of logging

    ## Wildcards to select database files
    wildcard = _("Datenbankdatei (*.db)|*.db|" \
                 "Alle Dateien (*.*) |*.*")

    ## Initialise dialog
    #
    #  \param self   The object pointer
    #  \param parent The parent object
    #  \param db_file
    #  \param log_enc
    #  \param log_level
    def __init__(self, parent, db_file, log_enc, log_level):
        # call parent constructor
        UIPreferences.UIPreferences.__init__(self, parent)

        # initialise internal variables
        self.db_file = db_file
        if log_enc in self.cc_Encoding.idx_obj.reverse:
            self.log_enc = log_enc
        else:
            self.log_enc = "ascii"
            logging.warning(
                _(u'Nicht untersttztes Encoding "%s" wird auf "ascii" zuckgesetzt'),
                log_enc,
                )
        if log_level in self.cc_LogLevel.idx_obj.reverse:
            self.log_level = log_level
        else:
            self.log_level = "info"
            logging.warning(
                _(u'Nicht untersttzte Geschwtzigkeit "%s" wird auf "info" zurckgesetzt'),
                log_level,
                )

        # fill dialog with data
        self.tc_DatabaseFile.SetValue(self.db_file)
        self.cc_LogLevel.SetSelection(self.log_level)
        self.cc_Encoding.SetSelection(self.log_enc)

        # disable creation of a new database is a store is already initialised
        if smm.database.store:
            self.bn_CreateNewDatabase.Enable(False)

        # bind additional events
        self.Bind(
            wx.EVT_BUTTON,
            self.OnCreateDatabase,
            self.bn_CreateNewDatabase
            )
        self.Bind(
            wx.EVT_BUTTON,
            self.OnSelectDatabase,
            self.bn_SelectDatabase
            )

    ## Create a new database
    #
    #  Query the database filename and create an database with an
    #  initial set of items
    def OnCreateDatabase(self, event):
        assert smm.database.store == None
        dlg = wx.FileDialog(
            None,
            _(u"Neue Datenbankdatei auswhlen"),
            defaultDir=os.path.dirname(self.db_file),
            defaultFile=_(u'smm.db'),
            wildcard=self.wildcard,
            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
            )
        if dlg.ShowModal() != wx.ID_OK:
            dlg.Destroy()
            return

        new_file = dlg.GetPath()
        dlg.Destroy()

        # check write permissions
        if os.path.exists(new_file) and not os.access(new_file, os.W_OK):
            wx.MessageBox(
                _(u'Die Datenbankdatei "%s" existiert bereits ist aber nicht schreibbar.' \
                  u'Bitte prfen Sie die Berechtigungen fr die Datei' % new_file),
                _(u'Fehler'),
                wx.OK | wx.ICON_ERROR,
                )
            return

        # truncate existing file if size is greather than 0
        if os.path.exists(new_file) and os.path.getsize(new_file):
            try:
                nf = open(new_file, 'wb')
                nf.truncate(0)
                nf.close()
            except (IOError, OSError), e:
                wx.MessageBox(
                    _(u'Die Datenbankdatei "%s" ist grer als 0 Byte ' \
                      u'und konnte vor dem Erzeugen der Struktur der ' \
                      u'Datenbank nicht auf eine Gre von 0 Byte ' \
                      u'reduziert werden.\n' \
                      u'Details zum Fehler stehen in der Logdatei.' % \
                      new_file),
                    _(u'Fehler'),
                    wx.OK | wx.ICON_ERROR,
                    )
                # Log error details
                logging.error("Konnte die Datei %s nicht auf 0 Byte krzen", new_file)
                logging.error("Fehlerdetails: %s", e)
                return

        # create a new database
        uri = u'sqlite:%s' % new_file
        smm.database.init_storm(uri)
        smm.db_setup.create_structure()
        smm.db_setup.load_initial_data()
        smm.database.close_storm()

        # use new created database?
        res = wx.MessageBox(
            _(u'Die Datenbank wurde erfolgreich angelegt.\n' \
              u'Soll die neu angelegte Datenbank genutzt werden?'),
            _(u'Neue Datenbank'),
            wx.YES_NO | wx.YES_DEFAULT,
            )
        if res == wx.YES:
            self.db_file = new_file
            self.tc_DatabaseFile.SetValue(self.db_file)
                

    ## Open a dialog to select an existing database file
    def OnSelectDatabase(self, event):
        dlg = wx.FileDialog(
            None,
            _(u"Datenbankdatei auswhlen"),
            defaultDir=os.path.dirname(self.db_file),
            defaultFile=_(u'smm.db'),
            wildcard=self.wildcard,
            style=wx.FD_OPEN|wx.FD_FILE_MUST_EXIST,
            )
        if dlg.ShowModal() != wx.ID_OK:
            dlg.Destroy()
            return

        new_file = dlg.GetPath()
        if new_file != self.db_file:
            self.db_file = new_file
            self.tc_DatabaseFile.SetValue(self.db_file)

        dlg.Destroy()

    ## Show the dialog modal and save the settings after afterwards
    #
    #  The insert data will stored in instance variables on success.
    def ShowModal(self, *args, **kwargs):
        res = UIPreferences.UIPreferences.ShowModal(self, *args, **kwargs)
        if res == wx.ID_OK:
            if self.tc_DatabaseFile.GetValue() != self.db_file:
                self.db_file = self.tc_DatabaseFile.GetValue()
            if self.cc_LogLevel.GetSelection() != self.log_level:
                self.log_level = self.cc_LogLevel.GetSelection()
            if self.cc_Encoding.GetSelection() != self.log_enc:
                self.log_enc = self.cc_Encoding.GetSelection()
        return res

## Open a HTML dialog and show a position report
#
#  \see DlgHtml, reports.createPositionReport()
class DlgShowPositionReport(wx.Dialog):

    ## Show the dialog modal
    #
    #  \param self The instance itself
    #  \param pos  The \ref smm.objects.Position "position" to report
    def ShowModal(self, pos):
        dlg_report = DlgHtml(self)
        dlg_report.SetTitle(
            _(u'Bericht ber %s' % helper.formatShare(pos.share))
        )
        buf = StringIO.StringIO()
        reports.createPositionReport(pos, buf)
        dlg_report.setHtml(buf.getvalue())
        dlg_report.Centre(wx.BOTH)
        res = dlg_report.ShowModal()
        dlg_report.Destroy()
        return res
