# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file gui/panel.py
#
# \brief Adaption of automatically generated wx.Notebook code
#
# \namespace gui::panel
# \brief The %panel classes presents various information to the user. 
#
# Every class shows a different set information with a special purpose.
# 
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
# 
# \par License:
# GPL version 2.0

import decimal
import logging
import sys
import wx
import wx.lib.pubsub as pubsub

import smm

import controls
import dialogs
import helper
import images
import UIBasePanelOpenPositionDetails
import UIBasePanelOpenPositionList
import UIBasePanelSummary
import UIBasePanelWatchList


## Panel to show a summary about the current deposit
class PanelSummary(UIBasePanelSummary.UIBasePanelSummary):

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        UIBasePanelSummary.UIBasePanelSummary.__init__(self, *args, **kwargs)

        # subscribe all changes
        pubsub.Publisher().subscribe(self.updateContent, 'smm')

        # fill panel
        self.updateContent('smm')

    ## \brief Query datastore and set appropriate values
    #
    #  This function receives messaged from the \c %smm channel
    #  and updates the %panel.
    #
    #  \todo Add calculating current volume
    #  \see \ref channels
    def updateContent(self, unused):  
        cur = smm.Currency.get(code='EUR')
        # collect data
        currencies = smm.Currency.getCount()
        exchanges  = smm.Exchange.getCount()
        fonds      = smm.Share.getCount(fonds=True)
        index      = smm.Share.getCount(index=True)
        other      = smm.Share.getCount(other=True)
        pos_closed = smm.Position.getCount(closed=True)
        pos_open   = smm.Position.getCount(open=True)
        pos_total  = smm.Position.getCount()
        quotes     = smm.DailyQuote.getCount()
        shares     = smm.Share.getCount(share=True)
        total      = smm.Share.getCount(total=True)
        trans      = smm.Transaction.getCount()
        volume     = helper.formatMoney(
            smm.Position.getTotalVolume(cur),
            cur
            )

        #  individual statistics
        self.tc_CurrentVolume.SetValue(volume)
        self.tc_OpenPositionCount.SetValue(u'%d' % pos_open)
        self.tc_ClosedPositionCount.SetValue(u'%d' % pos_closed)
        self.tc_TotalPositionCount.SetValue(u'%d' % pos_total)
        self.tc_TransactionCount.SetValue(u'%d' % trans)

        # general statistics
        self.tc_CurrencyCount.SetValue(u"%d" % currencies)
        self.tc_ExchangeCount.SetValue(u"%d" % exchanges)
        self.tc_FondCount.SetValue(u"%d" % fonds)
        self.tc_IndexCount.SetValue(u"%d" % index)
        self.tc_OtherCount.SetValue(u"%d" % other)
        self.tc_ShareCount.SetValue(u"%d" % shares)
        self.tc_TotalCount.SetValue(u"%d" % total)
        self.tc_DailyQuoteCount.SetValue(u"%d" % quotes)


## \brief Panel to show and maintain all currencies
#
#  Lists all smm.objects.Currency objects.
class PanelCurrency(controls.SortedListPanel):

    config_section = 'panel_currency'

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        controls.SortedListPanel.__init__(self, *args, **kwargs)

        # subscribe all changes
        pubsub.Publisher().subscribe(self.addCurrency,    'smm.currency.add')
        pubsub.Publisher().subscribe(self.changeCurrency, 'smm.currency.change')

    def createListHeader(self):
        self.list.InsertColumn(0, _(u'ISO-Code'))
        self.list.InsertColumn(1, _(u'Zeichen'))
        self.list.InsertColumn(2, _(u'Beschreibung'))

    ## \brief Fill the currency list
    def populateList(self):
        # use base functionality
        controls.SortedListPanel.populateList(self)

        # build new list
        currency_list = smm.Currency.get()
        for currency in currency_list:
            self._addCurrency(currency)
    
    ## \brief Add a new currency to the list
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def addCurrency(self, msg):
        currency = smm.Currency.get(id=msg.data)
        assert currency, 'Unknown id "%s"' % msg.data
        self._addCurrency(currency)
        self.list.ReSort()

    ## \brief A listed currency has been changed
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def changeCurrency(self, msg):
        self.removeEntry(msg.data)
        self.addCurrency(msg)

    ## \brief Add a smm.objects.Currency object to the list control
    #  \param self     The object instance itself
    #  \param currency The smm.objects.Currency object
    def _addCurrency(self, currency):
        code_show = currency.code
        code_sort = code_show.lower()
        sign_show = helper.formatCurrencySign(currency)
        sign_sort = sign_show.lower()
        desc_show = currency.description
        desc_sort = desc_show.lower()

        index = self.list.InsertStringItem(sys.maxint, code_show)
        self.list.SetStringItem(index, 1, sign_show)
        self.list.SetStringItem(index, 2, desc_show)

        # store data into itemDataMap
        self.list.itemDataMap[currency.id] = (code_sort, sign_sort, desc_sort)
        self.list.SetItemData(index, currency.id)


## \brief Panel to show and maintain all exchanges
#
#  Lists all smm.objects.Exchange objects.
class PanelExchange(controls.SortedListPanel):

    config_section = 'panel_exchange'
    initiallySort = 1

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        controls.SortedListPanel.__init__(self, *args, **kwargs)

        # subscribe all changes
        pubsub.Publisher().subscribe(self.addExchange,    'smm.exchange.add')
        pubsub.Publisher().subscribe(self.changeExchange, 'smm.exchange.change')

    def createListHeader(self):
        self.list.InsertColumn(0, _(u'Symbol'))
        self.list.InsertColumn(1, _(u'Beschreibung'))

    ## \brief Fill the exchange list
    def populateList(self):
        # use base functionality
        controls.SortedListPanel.populateList(self)

        # build new list
        exchange_list = smm.Exchange.get()
        for exchange in exchange_list:
            self._addExchange(exchange)

    ## \brief Add a new exchange to the list
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def addExchange(self, msg):
        exchange = smm.Exchange.get(id=msg.data)
        assert exchange, 'Unknown id "%s"' % msg.data
        self._addExchange(exchange)
        self.list.ReSort()

    ## \brief A listed exchange has been changed
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def changeExchange(self, msg):
        self.removeEntry(msg.data)
        self.addExchange(msg)

    ## \brief Add a smm.objects.Exchange object to the list control
    #
    #  \param self     The object instance itself
    #  \param exchange The smm.objects.Exchange object
    def _addExchange(self, exchange):
        desc_show = exchange.description
        desc_sort = desc_show.lower()
        symbol_show = exchange.symbol
        symbol_sort = symbol_show.lower()

        index = self.list.InsertStringItem(sys.maxint, symbol_show)
        self.list.SetStringItem(index, 1, desc_show)

        # store data into itemDataMap
        self.list.itemDataMap[exchange.id] = (symbol_sort, desc_sort)
        self.list.SetItemData(index, exchange.id)
    

## \brief Panel to show and maintain all shares
#
#  Lists all smm.objects.Share objects.
class PanelShare(controls.SortedListPanel):

    config_section = 'panel_share'
    initiallySort = 3

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        controls.SortedListPanel.__init__(self, *args, **kwargs)

        self.createPopupMenu()

        # subscribe all changes
        pubsub.Publisher().subscribe(self.addShare,    'smm.share.add')
        pubsub.Publisher().subscribe(self.changeShare, 'smm.share.change')
 
        # bind addtional events
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnEditShare, self.list)

    def createListHeader(self):
        self.list.InsertColumn(0, _(u'Symbol'))
        self.list.InsertColumn(1, _(u'ISIN'))
        self.list.InsertColumn(2, _(u'WKN'))
        self.list.InsertColumn(3, _(u'Name'))
        self.list.InsertColumn(4, _(u'Brsenplatz'))
        self.list.InsertColumn(5, _(u'Whrung'))
        self.list.InsertColumn(6, _(u'Gattung'))
        self.list.InsertColumn(7, _(u'Beobachten'))
        self.list.InsertColumn(8, _(u'Datenquelle'))
        self.list.InsertColumn(9, _(u'Notizen'))

    ## \brief Create the popup menu to do share releated actions
    #
    #  \see \ref OnShowPopup()
    def createPopupMenu(self):
        self.popupmenu = wx.Menu()

        # Add entry to trade
        trade = self.popupmenu.Append(-1, _(u'Handeln'))
        self.Bind(wx.EVT_MENU, self.OnTrade, trade)

        self.pmenu_watch = \
            self.popupmenu.Append(-1, _(u'Beobachten'))
        self.Bind(wx.EVT_MENU, self.OnWatch, self.pmenu_watch)

        self.pmenu_unwatch = \
            self.popupmenu.Append(-1, _(u'Nicht mehr beobachten'))
        self.Bind(wx.EVT_MENU, self.OnUnWatch, self.pmenu_unwatch)

        self.pmenu_newshare = \
            self.popupmenu.Append(-1, _(u'Neues Wertpapier anlegen'))
        self.Bind(wx.EVT_MENU, self.OnNewShare, self.pmenu_newshare)

        self.pmenu_editshare = \
            self.popupmenu.Append(-1, _(u'Wertpapier ndern'))
        self.Bind(wx.EVT_MENU, self.OnEditShare, self.pmenu_editshare)

        # Bind show menu event
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup, self.list)

    ## \brief Fill the share list
    def populateList(self):
        # use base functionality
        controls.SortedListPanel.populateList(self)

        # build new list
        share_list = smm.Share.get()
        if share_list:
            for share in share_list:
                self._addShare(share)

    ## \brief Add a new share to the list
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def addShare(self, msg):
        share = smm.Share.get(msg.data)
        assert share, 'Unknown id "%s"' % msg.data
        self._addShare(share)
        self.list.ReSort()

    ## \brief A listed share has been changed
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def changeShare(self, msg):
        self.removeEntry(msg.data)
        self.addShare(msg)

    ## \brief Add a smm.objects.Share object to the list control
    #
    #  \param self    The object instance itself
    #  \param share   The smm.objects.Share object
    def _addShare(self, share):
        currency_show = helper.formatCurrencyLong(share.currency)
        exchange_show = helper.formatExchange(share.exchange)
        isin_show     = share.isin or u''
        kind_show     = helper.formatShareKind(share)
        name_show     = share.name or u''
        notes_show    = helper.formatShareNotesShort(share)
        source_show   = smm.getPrimarySourceName(share)
        symbol_show   = share.symbol or u''
        if share.watch:
            watch_show = _(u'Ja')
        else:
            watch_show = _(u'Nein')
        wkn_show      = share.wkn or u''

        currency_sort = currency_show.lower()
        exchange_sort = exchange_show.lower()
        isin_sort     = isin_show.lower()
        kind_sort     = kind_show.lower()
        name_sort     = name_show.lower()
        notes_sort    = notes_show.lower()
        source_sort   = source_show.lower()
        symbol_sort   = symbol_show.lower()
        watch_sort    = watch_show.lower()
        wkn_sort      = wkn_show.lower()

        index = self.list.InsertStringItem(sys.maxint, symbol_show)
        self.list.SetStringItem(index, 1, isin_show)
        self.list.SetStringItem(index, 2, wkn_show)
        self.list.SetStringItem(index, 3, name_show)
        self.list.SetStringItem(index, 4, exchange_show)
        self.list.SetStringItem(index, 5, currency_show)
        self.list.SetStringItem(index, 6, kind_show)
        self.list.SetStringItem(index, 7, watch_show)
        self.list.SetStringItem(index, 8, source_show)
        self.list.SetStringItem(index, 9, notes_show)

        # store data into itemDataMap
        self.list.itemDataMap[share.id] = (
            symbol_sort,
            isin_sort,
            wkn_sort,
            name_sort,
            exchange_sort,
            currency_sort,
            kind_sort,
            watch_sort,
            source_sort,
            notes_sort
            )
        self.list.SetItemData(index, share.id)

    ## \brief Show a dialog to edit a share
    #
    #  \see \ref dialogs.DlgShare
    def OnEditShare(self, event):
        share_id = self.list.GetItemData(self.list.currentItem)
        share = smm.Share.get(share_id)
        assert share, 'Unknown id "%s"' % share_id

        # generate a new list of all transactions related to this share
        dialog = dialogs.DlgShare(self, share=share)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## \brief Show a dialog to create a new share
    #
    #  \see \ref dialogs.DlgShare
    def OnNewShare(self, event):
        dialog = dialogs.DlgShare(self)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## \brief Display the pop-up menu
    def OnShowPopup(self, event):
        share_id = self.list.GetItemData(self.list.currentItem)
        share = smm.Share.get(share_id)

        # enable / disable entries depending on the share
        watch = bool(share.watch)
        self.pmenu_watch.Enable(not watch)
        self.pmenu_unwatch.Enable(watch)

        # show menu
        pos = event.GetPosition()
        pos = self.ScreenToClient(pos)
        self.PopupMenu(self.popupmenu, pos)

    ## \brief Show a dialog to trade with the selected share
    #
    #  \see \ref dialogs.DlgTrade
    def OnTrade(self, event):
        share_id = self.list.GetItemData(self.list.currentItem)
        share = smm.Share.get(share_id)
        assert share, 'Unknown id "%s"' % share_id

        # generate a new list of all transactions related to this share
        dialog = dialogs.DlgTrade(self, share=share)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## Add a new watch list entry for the selected share
    def OnWatch(self, event):
        share_id = self.list.GetItemData(self.list.currentItem)
        share = smm.Share.get(share_id)

        if not share: 
            return

        name = helper.formatShare(share)

        dlg = wx.MessageDialog(
            None,
            _(u'Mchten Sie das Wertpapier "%s" beobachten?' % name),
            _(u'Beobachtungsliste'),
            wx.YES_NO | wx.YES_DEFAULT | wx.ICON_QUESTION
            )
        dlg.Centre(wx.BOTH)
        result = dlg.ShowModal()
        dlg.Destroy()

        if result == wx.ID_YES:
            share.createWatch()
            pubsub.Publisher().sendMessage('smm.watch.add', share.watch.id)

    ## Remove watch list entry for the selected share
    def OnUnWatch(self, event):
        share_id = self.list.GetItemData(self.list.currentItem)
        share = smm.Share.get(share_id)

        if not share: 
            return

        name = helper.formatShare(share)

        dlg = wx.MessageDialog(
            None,
            _(u'Mchten Sie das Wertpapier "%s" nicht mehr beobachten?' % name),
            _(u'Beobachtungsliste'),
            wx.YES_NO | wx.YES_DEFAULT | wx.ICON_QUESTION
            )
        result = dlg.ShowModal()
        dlg.Centre(wx.BOTH)
        dlg.Destroy()

        if result == wx.ID_YES:
            watch_id = share.watch.id
            share.watch.remove()
            pubsub.Publisher().sendMessage('smm.watch.remove', watch_id)


## \brief Panel to show and maintain all transactions
#
#  Lists all smm.objects.Transaction objects.
class PanelTransaction(controls.SortedListPanel):

    config_section = 'panel_transaction'

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        controls.SortedListPanel.__init__(self, *args, **kwargs)

        # subscribe all changes
        pubsub.Publisher().subscribe(self.addTransaction,    'smm.transaction.add')
        pubsub.Publisher().subscribe(self.changeTransaction, 'smm.transaction.change')

    def createListHeader(self):
        self.list.InsertColumn(0, _(u'Datum'))
        self.list.InsertColumn(1, _(u'Bestand Anzahl'), wx.LIST_FORMAT_RIGHT)
        self.list.InsertColumn(2, _(u'Wertpapier'))
        self.list.InsertColumn(3, _(u'Brsenplatz'))
        self.list.InsertColumn(4, _(u'Typ'))
        self.list.InsertColumn(5, _(u'Anteilswert'), wx.LIST_FORMAT_RIGHT)
        self.list.InsertColumn(6, _(u'Gebhr'), wx.LIST_FORMAT_RIGHT)
        self.list.InsertColumn(7, _(u'Volumen'), wx.LIST_FORMAT_RIGHT)

    ## \brief Fill the transaction list
    def populateList(self):
        # use base functionality
        controls.SortedListPanel.populateList(self)

        # build new list
        transaction_list = smm.Transaction.get()
        for transaction in transaction_list:
            self._addTransaction(transaction)

    ## \brief Add a new transaction to the list
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def addTransaction(self, msg):
        transaction = smm.Transaction.get(id=msg.data)
        assert transaction, 'Unknown id "%s"' % msg.data
        self._addTransaction(transaction)
        self.list.ReSort()

    ## \brief A listed transaction has been changed
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def changeTransaction(self, msg):
        self.removeEntry(msg.data)
        self.addTransaction(msg)

    ## \brief Add a smm.objects.Transaction object to the list control
    #
    #  \param self         The object instance itself
    #  \param transaction  The smm.objects.Transaction object
    def _addTransaction(self, transaction):
        # aliases
        currency = transaction.position.currency

        # initialise internal variables
        count_sort    = transaction.count
        count_show    = helper.formatDecimal(count_sort, 3)
        date_show     = helper.formatDate(transaction.date)
        date_sort     = unicode(transaction.date)
        exchange_show = helper.formatExchange(transaction.exchange)
        exchange_sort = exchange_show.lower()
        fee_sort      = transaction.fee
        fee_show      = helper.formatMoney(fee_sort, currency)
        kind_show     = helper.formatTransactionKind(transaction)
        kind_sort     = kind_show.lower()
        share_show    = helper.formatShare(transaction.position.share)
        share_sort    = share_show.lower()
        value_sort    = transaction.value
        value_show    = helper.formatMoney(value_sort, currency)
        volume_sort   = transaction.getVolume()
        volume_show   = helper.formatMoney(volume_sort, currency)

        index = self.list.InsertStringItem(sys.maxint, date_show)
        self.list.SetStringItem(index, 1, count_show)
        self.list.SetStringItem(index, 2, share_show)
        self.list.SetStringItem(index, 3, exchange_show)
        self.list.SetStringItem(index, 4, kind_show)
        self.list.SetStringItem(index, 5, value_show)
        self.list.SetStringItem(index, 6, fee_show)
        self.list.SetStringItem(index, 7, volume_show)

        # store data into itemDataMap
        self.list.itemDataMap[transaction.id] = (
            date_sort,
            count_sort,
            share_sort,
            exchange_sort,
            kind_sort,
            value_sort,
            fee_sort,
            volume_sort
            )
        self.list.SetItemData(index, transaction.id)


## \brief Panel to show and maintain all indices
#
#  Lists all smm.objects.Share.kind == 'index' objects.
#
#  \note An index is a special kind of a Share!
class PanelIndex(controls.SortedListPanel):

    config_section = 'panel_index'
    initiallySort = 3

    ## \brief Initialise widget
    def __init__(self, *args, **kwargs):
        # initialise base class
        controls.SortedListPanel.__init__(self, *args, **kwargs)

        self.createPopupMenu()

        # subscribe all changes
        pubsub.Publisher().subscribe(self.addIndex, 'smm.share.add')
        pubsub.Publisher().subscribe(self.changeIndex, 'smm.share.change')
 
        # bind addtional events
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnEditIndex, self.list)

    def createListHeader(self):
        self.list.InsertColumn(0, _(u'Symbol'))
        self.list.InsertColumn(1, _(u'ISIN'))
        self.list.InsertColumn(2, _(u'WKN'))
        self.list.InsertColumn(3, _(u'Name'))

    ## \brief Create the popup menu to do share releated actions
    #
    #  \see \ref OnShowPopup()
    def createPopupMenu(self):
        self.popupmenu = wx.Menu()

        self.pmenu_newindex = \
            self.popupmenu.Append(-1, _(u'Neuen Index anlegen'))
        self.Bind(wx.EVT_MENU, self.OnNewIndex, self.pmenu_newindex)

        self.pmenu_editindex = \
            self.popupmenu.Append(-1, _(u'Index ndern'))
        self.Bind(wx.EVT_MENU, self.OnEditIndex, self.pmenu_editindex)

        # Bind show menu event
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup, self.list)


    ## \brief Fill the index list
    def populateList(self):
        # use base functionality
        controls.SortedListPanel.populateList(self)

        # build new list
        index_list = smm.Share.get(index=True)
        if index_list:
            for index in index_list:
                self._addIndex(index)

    ## \brief Add a new index to the list
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def addIndex(self, msg):
        index = smm.Share.get(msg.data)
        assert index, 'Unknown id "%s"' % msg.data
        self._addIndex(index)
        self.list.ReSort()

    ## \brief A listed index has been changed
    #
    #  \param self The object instance itself
    #  \param msg  \c pubsub.Message
    def changeIndex(self, msg):
        self.removeEntry(msg.data)
        self.addIndex(msg)

    ## \brief Add a smm.objects.Share object to the list control
    #
    #  \param self    The object instance itself
    #  \param share   The smm.objects.Share object
    def _addIndex(self, share):
        symbol_show = share.symbol or u''
        symbol_sort = symbol_show.lower()
        isin_show   = share.isin or u''
        isin_sort   = isin_show.lower()
        wkn_show    = share.wkn or u''
        wkn_sort    = wkn_show.lower()
        name_show   = share.name or u''
        name_sort   = name_show.lower()

        index = self.list.InsertStringItem(sys.maxint, symbol_show)
        self.list.SetStringItem(index, 1, isin_show)
        self.list.SetStringItem(index, 2, wkn_show)
        self.list.SetStringItem(index, 3, name_show)

        # store data into itemDataMap
        self.list.itemDataMap[share.id] = \
            (symbol_sort, isin_sort, wkn_sort, name_sort)
        self.list.SetItemData(index, share.id)

    ## \brief Show a dialog to edit a share
    #
    #  \see \ref dialogs.DlgShare
    def OnEditIndex(self, event):
        index_id = self.list.GetItemData(self.list.currentItem)
        index = smm.Share.get(index_id)
        assert index, 'Unknown id "%s"' % index_id

        # generate a new list of all transactions related to this share
        dialog = dialogs.DlgShare(self, share=index)
        dialog.setFixedShareKind('index')
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## \brief Show a dialog to create a new index
    #
    #  \see \ref dialogs.DlgShare
    def OnNewIndex(self, event):
        dialog = dialogs.DlgShare(self)
        dialog.setFixedShareKind('index')
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## \brief Display the pop-up menu
    def OnShowPopup(self, event):
        # show menu
        pos = event.GetPosition()
        pos = self.ScreenToClient(pos)
        self.PopupMenu(self.popupmenu, pos)



## \brief Panel to show all open positions
class PanelOpenPositionList( \
    UIBasePanelOpenPositionList.UIBasePanelOpenPositionList,
    controls.GeneralPanelConfig):

    ## \var config_section
    #  \brief Section in the configuration that contains the current look
    config_section = 'panel_positionlist'

    config_items = {
        # attr    type    
        'lc_OpenPosition': 'ListCtrl',
        'lc_Transaction':  'ListCtrl',
        'sw_OpenPosition': 'SplitterWindow',
        }

    ## \var lc_OpenPosition
    #  \brief List of all open positions

    ## \var transaction_list
    #  \brief Instance to simplify controlling of the list of
    #         transactions related to the current selected share.
    #  \see controls.TransactionListSmall

    ## \var sw_OpenPosition
    #  \brief Splitter window used as main widget in this %panel.

    ## \var green_arrow_idx
    #  \brief Position of the green arrow image inside the ListCtrl image list

    ## \var yellow_arrow_idx
    #  \brief Position of the yellow arrow image inside the ListCtrl image list

    ## \var red_arrow_idx
    #  \brief Position of the red arrow image inside the ListCtrl image list

    ## \var popupmenu
    #  \brief Reference to the pop-up menu
    #
    #  \see createPopupMenu() as well as OnShowPopup()

    ## \brief Initialise the open position frame
    def __init__(self, *args, **kwds):
        kwds["style"] = wx.TAB_TRAVERSAL
        # initialise base classes
        UIBasePanelOpenPositionList.UIBasePanelOpenPositionList.__init__(
            self,
            *args,
            **kwds
            )
        controls.GeneralPanelConfig.__init__(self)

        # create objects
        self.sw_OpenPosition.SetMinimumPaneSize(50)

        # create lists
        self.createListOpenPosition()
        self.lc_OpenPosition.enableColumnSorter()
        self.transaction_list = controls.TransactionListSmall(self.lc_Transaction)

        # add new images to open position list
        self.green_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallGreenArrowBitmap())
        self.yellow_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallYellowArrowBitmap())
        self.red_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallRedArrowBitmap())

        # fill and sort open positions
        self.populateOpenPosition()
        self.lc_OpenPosition.SortListItems(col=1)

        self.createPopupMenu()

        # restore saved settings
        wx.FutureCall(500, self.restoreConfig)

        # bind all events to the event handler
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lc_OpenPosition)

        # subscribe all changes
        pubsub.Publisher().subscribe(self.updateContent, 'smm')

    ## \brief Create a header only list for all open positions
    def createListOpenPosition(self):
        # configure the already created list
        self.lc_OpenPosition.InsertColumn(0, _(u''))
        self.lc_OpenPosition.InsertColumn(1, _(u'Name'))
        self.lc_OpenPosition.InsertColumn(2, _(u'Bestand Anteile'), wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(3, _(u'Bestand Euro'), wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(4, _(u'Anteilswert'), wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(5, _(u'Kursdatum'),   wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(6, _(u'GuV absolut'), wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(7, _(u'GuV relativ'), wx.LIST_FORMAT_RIGHT)
        self.lc_OpenPosition.InsertColumn(8, _(u'Kommentar'))

    ## \brief Create the popup menu to do share releated actions
    def createPopupMenu(self):
        self.popupmenu = wx.Menu()

        # Add entry to trade
        trade = self.popupmenu.Append(-1, _(u'Handeln'))
        self.Bind(wx.EVT_MENU, self.OnTrade, trade)

        # Bind show menu event
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnShowPopup, self.lc_OpenPosition)

    ## \brief Creates a list with all open positions
    def populateOpenPosition(self):
        # delete existing entries
        self.lc_OpenPosition.DeleteAllItems()

        # build new list
        position_list = smm.Position.get(open=True)
        for position in position_list:
            self._addPosition(position)

    ## \brief Add a new single smm.objects.OpenPosition object to the list
    #
    #  \param self  The object instance itself
    #  \param pos   The smm.objects.Position object
    def _addPosition(self, pos):
        # aliases
        share    = pos.share
        currency = pos.currency

        # initialise internal variables
        count_sort = pos.getShareCount()
        count_show = helper.formatDecimal(count_sort, 3)
        date_show  = u''
        date_sort  = u''
        perf_abs_show = u''
        perf_abs_sort = decimal.Decimal(0)
        perf_rel_show = u''
        perf_rel_sort = decimal.Decimal(0)
        quote_show = u''
        quote_sort = decimal.Decimal(0)
        trend_sort = u''
        volume_show = u''
        volume_sort = decimal.Decimal(0)
        trend_img_idx = -1

        # collect data of the open position
        name_show    = share.name or u''
        name_sort    = name_show.lower()
        comment_show = helper.formatShareNotesShort(share)
        comment_sort = comment_show.lower()
        volume_sort  = pos.getVolume()
        volume_show  = helper.formatMoney(volume_sort, currency)

        quote, date = pos.getLastQuote()
        if quote:
            date_show  = helper.formatDate(date)
            date_sort  = unicode(date)
            quote_show = helper.formatMoney(quote, currency)
            quote_sort = quote

        perf_abs, perf_rel = pos.getPerf()
        if perf_abs:
            if perf_abs > 0:
                trend_img_idx = self.green_arrow_idx
                trend_sort = _(u'steigend')
            elif perf_abs == 0:
                trend_img_idx = self.yellow_arrow_idx
                trend_sort = _(u'neutral')
            elif perf_abs < 0:
                trend_img_idx = self.red_arrow_idx
                trend_sort = _(u'fallend')
            perf_abs_show = helper.formatMoney(perf_abs, currency)
            perf_abs_sort = perf_abs
            perf_rel_show = helper.formatPercent(perf_rel, 2)
            perf_rel_sort = perf_rel

        # fill row
        index = self.lc_OpenPosition.InsertImageItem(sys.maxint, trend_img_idx)

        self.lc_OpenPosition.SetStringItem(index, 1, name_show)
        self.lc_OpenPosition.SetStringItem(index, 2, count_show)
        self.lc_OpenPosition.SetStringItem(index, 3, volume_show)
        self.lc_OpenPosition.SetStringItem(index, 4, quote_show)
        self.lc_OpenPosition.SetStringItem(index, 5, date_show)
        self.lc_OpenPosition.SetStringItem(index, 6, perf_abs_show)
        self.lc_OpenPosition.SetStringItem(index, 7, perf_rel_show)
        self.lc_OpenPosition.SetStringItem(index, 8, comment_show)

        # store data into itemDataMap
        self.lc_OpenPosition.itemDataMap[pos.id] = (
            trend_sort,
            name_sort,
            count_sort,
            volume_sort,
            quote_sort,
            date_sort,
            perf_abs_sort,
            perf_rel_sort,
            comment_sort
            )
        self.lc_OpenPosition.SetItemData(index, pos.id)
        
    ## \brief Event handler to update the transaction list on selecting
    #         an open position
    def OnItemSelected(self, event):
        # get share from selected row
        index = event.m_itemIndex
        pos_id = self.lc_OpenPosition.GetItemData(index)
        pos = smm.Position.get(id=pos_id)

        # generate a new list of all transactions related to this share
        if pos:
            self.transaction_list.populate(pos.transactions)
        else:
            logging.warning(
                _(u'PanelOpenPositionList.OnItemSelected(): Unbekannte Id "%s"'),
                pos_id
                )
            self.transaction_list.clean()

        # continue event processing
        event.Skip()

    ## \brief Display the pop-up menu
    def OnShowPopup(self, event):
        pos = event.GetPosition()
        pos = self.ScreenToClient(pos)
        self.PopupMenu(self.popupmenu, pos)

    ## \brief Show a dialog to trade with the selected share
    #
    #  \see \ref dialogs.DlgTrade
    def OnTrade(self, event):
        pos_id = self.lc_OpenPosition.GetItemData(
            self.lc_OpenPosition.currentItem
            )
        pos = smm.Position.get(id=pos_id)
        assert pos, 'Unknown id "%s"' % pos_id

        # generate a new list of all transactions related to this share
        dialog = dialogs.DlgTrade(self, position=pos)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

        # continue event processing
        event.Skip()

    ## \brief Update the panel if new quotes are available
    #
    #  \see \ref channels
    def updateContent(self, unused):
        # fill and sort open positions
        self.populateOpenPosition()
        self.lc_OpenPosition.SortListItems(col=1)

        # update transaction list too -> means delete existing entries
        self.transaction_list.clean()


## \brief Panel to show all open positions
class PanelOpenPositionDetails( \
    UIBasePanelOpenPositionDetails.UIBasePanelOpenPositionDetails,
    controls.GeneralPanelConfig):

    ## \var config_section
    #  \brief Section in the configuration that contains the current look
    config_section = 'panel_positiondetails'

    config_items = {
        # attr    type    
        'lc_OpenPosition': 'ListCtrl',
        'lc_Transaction':  'ListCtrl',
        'sw_OpenPosition': 'SplitterWindow',
        }

    ## \var lc_OpenPosition
    #  \brief List of all open positions

    ## \var transaction_list
    #  \brief Instance to simplify controlling of the list of
    #         transactions related to the current selected share.
    #  \see controls.TransactionListSmall

    ## \var sw_OpenPosition
    #  \brief Splitter window used as main widget in this %panel.

    ## \var green_arrow_idx
    #  \brief Position of the green arrow image inside the ListCtrl image list

    ## \var yellow_arrow_idx
    #  \brief Position of the yellow arrow image inside the ListCtrl image list

    ## \var red_arrow_idx
    #  \brief Position of the red arrow image inside the ListCtrl image list

    ## \var position
    #  \brief Position whose details are shown

    ## \brief Initialise the open position frame
    def __init__(self, *args, **kwds):
        
        # initialise base classes
        UIBasePanelOpenPositionDetails.UIBasePanelOpenPositionDetails.__init__(self, *args, **kwds)
        controls.GeneralPanelConfig.__init__(self)

        self.sw_OpenPosition.SetMinimumPaneSize(50)

        # initialise variables
        self.position = None
  
        # create lists
        self.createListOpenPosition()
        self.lc_OpenPosition.enableColumnSorter()
        self.transaction_list = controls.TransactionListSmall(self.lc_Transaction)
  
        # add new images to open position list
        self.green_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallGreenArrowBitmap())
        self.yellow_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallYellowArrowBitmap())
        self.red_arrow_idx = \
            self.lc_OpenPosition.img_list.Add(images.getSmallRedArrowBitmap())
  
        # fill and sort open positions
        self.populateOpenPosition()
        self.showDetails()
        self.lc_OpenPosition.SortListItems(col=1)
  
        # restore saved settings
        wx.FutureCall(500, self.restoreConfig)
  
        # bind all events to the event handler
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lc_OpenPosition)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.bn_cancel)
        self.Bind(wx.EVT_BUTTON, self.OnSave, self.bn_save)
        self.Bind(wx.EVT_BUTTON, self.OnTrade, self.bn_trade)

        # subscribe for new quotes
        pubsub.Publisher().subscribe(self.updateContent, 'smm')

    ## \brief Create a header only list for all open positions
    def createListOpenPosition(self):
        # configure the already created list
        self.lc_OpenPosition.InsertColumn(0, _(u''))
        self.lc_OpenPosition.InsertColumn(1, _(u'Name'))

    ## \brief Creates a list with all open positions
    def populateOpenPosition(self):
        # delete existing entries
        self.lc_OpenPosition.DeleteAllItems()

        # build new list
        position_list = smm.Position.get(open=True)
        for pos in position_list:
            self._addPosition(pos)

    ## \brief Add a new single smm.objects.OpenPosition object to the list
    #
    #  \param self  The object instance itself
    #  \param pos   The smm.objects.Position object
    def _addPosition(self, pos):
        # initialise internal variables
        trend_sort = u''
        trend_img_idx = -1

        # collect data of the open position
        name_show = pos.share.name or u''
        name_sort = name_show.lower()
        perf_abs = pos.getPerf()[0]
        if isinstance(perf_abs, decimal.Decimal):
            if perf_abs > 0:
                trend_img_idx = self.green_arrow_idx
                trend_sort = _(u'steigend')
            elif perf_abs == 0:
                trend_img_idx = self.yellow_arrow_idx
                trend_sort = _(u'neutral')
            elif perf_abs < 0:
                trend_img_idx = self.red_arrow_idx
                trend_sort = _(u'fallend')

        # fill row
        index = self.lc_OpenPosition.InsertImageItem(sys.maxint, trend_img_idx)
        self.lc_OpenPosition.SetStringItem(index, 1, name_show)

        # store data into itemDataMap
        self.lc_OpenPosition.itemDataMap[pos.id] = (trend_sort, name_sort)
        self.lc_OpenPosition.SetItemData(index, pos.id)

    ## \brief Shows the details of a selected share
    #
    #  \todo Rework usage of \c currency variable 
    def showDetails(self):

        # update button state depending if a share is selected or not
        self.bn_cancel.Enable(self.position != None)
        self.bn_save.Enable(self.position != None)
        self.bn_trade.Enable(self.position != None)

        if not self.position:
            avg_buy    = u''
            count      = u''
            date       = u''
            notes      = u''
            profit_abs = u''
            profit_rel = u''
            quote      = u''
            volume     = u''
            trend_img_idx = None

            self.pn_PositionDetails.cc_Share.Select(None)
            self.pn_PositionDetails.cc_Currency.Select(None)

        else:
            # aliases
            currency = self.position.currency

            count    = self.position.getShareCount()
            notes    = helper.formatShareNotesLong(self.position)
            trend_img_idx = -1

            # collect data of the open position
            volume = helper.formatMoney(
                self.position.getVolume(),
                currency
                )
    
            quote, date = self.position.getLastQuote()
            if quote:
                date  = helper.formatDate(date)
                quote = helper.formatMoney(quote, currency)
            else:
                date = u''
                quote = u''

            profit_abs, profit_rel = self.position.getPerf()
            if profit_abs != None:
                if profit_abs > 0:
                    trend_img_idx = self.green_arrow_idx
                elif profit_abs == 0:
                    trend_img_idx = self.yellow_arrow_idx
                elif profit_abs < 0:
                    trend_img_idx = self.red_arrow_idx
                profit_abs = helper.formatMoney(profit_abs, currency)
                profit_rel = helper.formatPercent(profit_rel, 2)
            else:
                profit_abs = u''
                profit_rel = u''
    
            avg_buy  = helper.formatMoney(self.position.getAvgBuy(), currency)
            count    = helper.formatDecimal(count, 3)

            self.pn_PositionDetails.cc_Share.Select(self.position.share)
            self.pn_PositionDetails.cc_Currency.Select(currency)
    
        # fill dialog
        self.pn_PositionDetails.tc_Notes.SetValue(notes)
        self.tc_Buy.SetValue(avg_buy)
        self.tc_Count.SetValue(count)
        self.tc_Date.SetValue(date)
        self.tc_ProfitAbsolute.SetValue(profit_abs)
        self.tc_ProfitRelative.SetValue(profit_rel)
        self.tc_Quote.SetValue(quote)
        self.tc_Volume.SetValue(volume)


        if trend_img_idx == None:
            self.bm_Trend.Hide()
        else:
            self.bm_Trend.SetBitmap(
                self.lc_OpenPosition.img_list.GetBitmap(trend_img_idx)
                )
            self.bm_Trend.Show()

        # show proper transactions
        if self.position:
            self.transaction_list.populate(self.position.transactions)
        else:
            self.transaction_list.clean()

    ## \brief Event handler to update the transaction list on selecting
    #         an open position
    def OnItemSelected(self, event):
        # get share from selected row
        index = event.m_itemIndex
        pos_id = self.lc_OpenPosition.GetItemData(index)
        pos = smm.Position.get(id=pos_id)
        assert pos, 'Unknown id "%s"' % pos_id

        # generate a new list of all transactions related to this share
        self.position = pos
        self.showDetails()

        # continue event processing
        event.Skip()

    ## \brief Update the panel if new quotes are available
    #
    #  \see \ref channels
    def updateContent(self, unused):
        # check if selected share is already a open position
        if self.position:
            position_list = smm.Position.get(open=True)
            if self.position not in position_list:
                self.position = None
            
        # fill and sort shares by name initially
        self.populateOpenPosition()
        self.lc_OpenPosition.SortListItems(col=1)
      
        # update share details
        self.showDetails()

    ## \brief Revert changes
    def OnCancel(self, event):
        self.showDetails()

    ## \brief Write changes back to the share
    def OnSave(self, event):
        update_share = self.position.share != self.pn_PositionDetails.cc_Share.GetSelection()
        self.position.update(
            currency=self.pn_PositionDetails.cc_Currency.GetSelection(),
            share=self.pn_PositionDetails.cc_Share.GetSelection(),
            notes=self.pn_PositionDetails.tc_Notes.GetValue()
            )
        if update_share:
            self.lc_OpenPosition.SetStringItem(
                self.lc_OpenPosition.currentItem,
                1,
                self.position.share.name
                )


    ## \brief Show a dialog to trade with the selected share
    #
    #  \see \ref dialogs.DlgTrade
    def OnTrade(self, event):
        dialog = dialogs.DlgTrade(self, 'buy', position=self.position)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()


## Panel to show all open positions
class PanelWatchList( \
    UIBasePanelWatchList.UIBasePanelWatchList,
    controls.GeneralPanelConfig):

    ## \var config_section
    #  \brief Section in the configuration that contains the current look
    config_section = 'panel_watchlist'

    config_items = {
        # attr    type    
        'lc_DailyQuotes':   'ListCtrl',
        'lc_WatchPosition': 'ListCtrl',
        'sw_WatchList':     'SplitterWindow',
        }

    ## \var lc_DailyQuotes
    #  Daily quotes of the watched share

    ## \var lc_WatchPosition
    #  List of all watched shares

    ## \var sw_WatchList
    #  Splitter window used as main widget in this %panel.

    ## \var watch
    #  Watch list entry


    ## Initialise the watch list frame
    def __init__(self, *args, **kwds):
        
        # initialise base classes
        UIBasePanelWatchList.UIBasePanelWatchList.__init__(self, *args, **kwds)
        controls.GeneralPanelConfig.__init__(self)

        self.sw_WatchList.SetMinimumPaneSize(50)

        # initialise variables
        self.watch = None

        # create lists
        self.createWatchList()
        self.lc_WatchPosition.enableColumnSorter()
        self.createQuoteList()
        self.lc_DailyQuotes.enableColumnSorter()

        # fill and sort open positions
        self.populateWatchList()
        self.showDetails()
        self.lc_WatchPosition.SortListItems(col=0)

        # restore saved settings
        wx.FutureCall(1000, self.restoreConfig)

        # bind all events to the event handler
        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.bn_cancel)
        self.Bind(wx.EVT_BUTTON, self.OnDelete, self.bn_delete)
        self.Bind(wx.EVT_BUTTON, self.OnSave, self.bn_save)
        self.Bind(wx.EVT_BUTTON, self.OnTrade, self.bn_trade)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lc_WatchPosition)

        # subscribe for new quotes
        pubsub.Publisher().subscribe(self.updateAllContent, 'smm.watch')
        pubsub.Publisher().subscribe(self.updateQuoteList,  'smm.quote')


    ## \brief Create a header only list for all open positions
    def createWatchList(self):
        # configure the already created list
        self.lc_WatchPosition.InsertColumn(0, _(u'Name'))


    ## \brief Create a header only list for daily quotes
    def createQuoteList(self):
        # configure the already created list
        self.lc_DailyQuotes.InsertColumn(0, _(u'Datum'))
        self.lc_DailyQuotes.InsertColumn(1, _(u'Brsenplatz'))
        self.lc_DailyQuotes.InsertColumn(
            2,
            _(u'Erffnungskurs'),
            wx.LIST_FORMAT_RIGHT
            )
        self.lc_DailyQuotes.InsertColumn(
            3,
            _(u'Schlukurs'),
            wx.LIST_FORMAT_RIGHT
            )
        self.lc_DailyQuotes.InsertColumn(
            4,
            _(u'Tageshoch'),
            wx.LIST_FORMAT_RIGHT
            )
        self.lc_DailyQuotes.InsertColumn(
            5,
            _(u'Tagestief'),
            wx.LIST_FORMAT_RIGHT
            )
        self.lc_DailyQuotes.InsertColumn(
            6,
            _(u'Volumen'),
            wx.LIST_FORMAT_RIGHT
            )
        self.lc_DailyQuotes.InsertColumn(
            7,
            _(u'Ereignis'),
            )


    ## \brief Creates a list with all open positions
    def populateWatchList(self):
        # delete existing entries
        self.lc_WatchPosition.DeleteAllItems()

        # build new list
        watch_list = smm.Watch.get()
        for watch in watch_list:
            self._addPosition(watch)

        # check if watch list entry is still member of the watch list
        if self.watch and self.watch.id not in self.lc_WatchPosition.itemDataMap:
            self.watch = None


    ## \brief List all quotes refered by the selectet share
    def populateQuoteList(self):
        # delete existing entries
        self.lc_DailyQuotes.DeleteAllItems()

        # return if not share selected
        if not self.watch:
            return

        # build new list
        quote_list = smm.DailyQuote.get(share=self.watch.share)
        for quote in quote_list:
            self._addQuote(quote)

        # sort new filled list initially
        if quote_list:
            self.lc_DailyQuotes.SortListItems(col=0, ascending=0)


    ## \brief Add a new watch list entry to the list
    #
    #  \param self   The object instance itself
    #  \param watch  The \ref smm.objects.Watch "watch list entry"
    def _addPosition(self, watch):
        name_show = watch.share.name or u''
        name_sort = name_show.lower()
        index = self.lc_WatchPosition.InsertStringItem(sys.maxint, name_show)

        # store data into itemDataMap
        self.lc_WatchPosition.itemDataMap[watch.id] = (name_sort,)
        self.lc_WatchPosition.SetItemData(index, watch.id)


    ## \brief Add a new quote to the list
    #
    #  \param self    The object instance itself
    #  \param quote   The smm.objects.DailyQuote object
    def _addQuote(self, quote):
        close_show  = helper.formatMoney(quote.close, quote.currency)
        close_sort  = quote.close
        date_show   = helper.formatDate(quote.date)
        date_sort   = unicode(quote.date)
        down_show   = helper.formatMoney(quote.low, quote.currency)
        down_sort   = quote.low
        exchange    = helper.formatExchange(quote.exchange)
        open_show   = helper.formatMoney(quote.open, quote.currency)
        open_sort   = quote.open
        top_show    = helper.formatMoney(quote.high, quote.currency)
        top_sort    = quote.high
        event_show  = helper.formatEvent(quote.event)
        event_sort  = event_show.lower()
        if quote.volume:
            volume_show = u'%d' % quote.volume
            volume_sort = quote.volume
        else:
            volume_show = u''
            volume_sort = 0

        index = self.lc_DailyQuotes.InsertStringItem(sys.maxint, date_show)
        self.lc_DailyQuotes.SetStringItem(index, 1, exchange)
        self.lc_DailyQuotes.SetStringItem(index, 2, open_show)
        self.lc_DailyQuotes.SetStringItem(index, 3, close_show)
        self.lc_DailyQuotes.SetStringItem(index, 4, top_show)
        self.lc_DailyQuotes.SetStringItem(index, 5, down_show)
        self.lc_DailyQuotes.SetStringItem(index, 6, volume_show)
        self.lc_DailyQuotes.SetStringItem(index, 7, event_show)

        # store data into itemDataMap
        self.lc_DailyQuotes.itemDataMap[quote.id] = (
            date_sort,
            exchange,
            open_sort,
            close_sort,
            top_sort,
            down_sort,
            volume_sort,
            event_sort,
            )
        self.lc_DailyQuotes.SetItemData(index, quote.id)


    ## \brief Shows the details of a selected share
    def showDetails(self):
        # update button state depending if a share is selected or not
        self.bn_cancel.Enable(self.watch != None)
        self.bn_delete.Enable(self.watch != None)
        self.bn_save.Enable(self.watch != None)
        self.bn_trade.Enable(self.watch != None)

        if not self.watch:
            notes   = u''

            date_created = wx.DateTime_Today()
            date_expire  = wx.DateTime_Today()

            self.pn_WatchDetails.dp_expire.Enable(False)
            self.pn_WatchDetails.cc_Share.Select(None)

        else:
            watch = self.watch
            notes = helper.formatShareNotesLong(watch)

            date_created = helper.pydate2wxdate(watch.created)

            if watch.expire:
                self.pn_WatchDetails.cb_expire.SetValue(True)
                self.pn_WatchDetails.dp_expire.Enable(True)
                date_expire = helper.pydate2wxdate(watch.expire)
            else:
                self.pn_WatchDetails.cb_expire.SetValue(False)
                self.pn_WatchDetails.dp_expire.Enable(False)
                date_expire  = wx.DateTime_Today()

            self.pn_WatchDetails.cc_Share.Select(self.watch.share)

        # fill dialog
        self.pn_WatchDetails.tc_Notes.SetValue(notes)

        self.pn_WatchDetails.dp_created.SetValue(date_created)
        self.pn_WatchDetails.dp_expire.SetValue(date_expire)

        # fill quotes list
        self.populateQuoteList()


    ## \brief Event handler to update the transaction list on selecting
    #         an open position
    def OnItemSelected(self, event):
        # get share from selected row
        index = event.m_itemIndex
        watch_id = self.lc_WatchPosition.GetItemData(index)
        watch = smm.Watch.get(watch_id)
        assert watch, 'Unknown id "%s"' % watch_id

        # generate a new list of all transactions related to this share
        self.watch = watch
        self.showDetails()

        # continue event processing
        event.Skip()


    ## \brief Full panel update
    #
    #  \see \ref channels
    def updateAllContent(self, unused):
        # fill and sort shares by name initially
        self.populateWatchList()
        self.lc_WatchPosition.SortListItems(col=0)

        # update share details
        self.showDetails()


    ## \brief Quote list update
    #
    #  \see \ref channels
    def updateQuoteList(self, unused):
        self.populateQuoteList()


    ## \brief Revert changes
    def OnCancel(self, event):
        self.showDetails()


    ## \brief Delete watch list entry
    def OnDelete(self, event):
        if not self.watch:
            return

        name = helper.formatShare(self.watch.share)

        dlg = wx.MessageDialog(
            None,
            _(u'Mchten Sie das Wertpapier "%s" weiter beobachten?' % name),
            _(u'Beobachtungsliste'),
            wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION
            )
        dlg.Centre(wx.BOTH)
        result = dlg.ShowModal()
        dlg.Destroy()

        if result == wx.ID_NO:
            watch_id = self.watch.id
            self.watch.remove()
            self.watch = None
            pubsub.Publisher().sendMessage('smm.watch.remove', watch_id)


    ## \brief Write changes back
    #
    #  The watch list entry as well as the share will be updated
    def OnSave(self, event):
        if self.pn_WatchDetails.cb_expire.GetValue():
            expire = helper.wxdate2pydate(self.pn_WatchDetails.dp_expire.GetValue())
        else:
            expire = None

        self.watch.update(
            created=helper.wxdate2pydate(self.pn_WatchDetails.dp_created.GetValue()),
            expire=expire,
            notes=self.pn_WatchDetails.tc_Notes.GetValue(),
            )


    ## \brief Show a dialog to trade with the selected share
    #
    #  \see \ref dialogs.DlgTrade
    def OnTrade(self, event):
        dialog = dialogs.DlgTrade(self, 'buy', share=self.watch.share)
        dialog.Centre(wx.BOTH)
        dialog.ShowModal()
        dialog.Destroy()

