# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2010 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file smm/quote/raiffeisen.py
#
# Functionality to query the finance %database of Raiffeisen Bankengruppe.
#
# \namespace smm::quote::raiffeisen
# Functionality to query the finance %database of Raiffeisen Bankengruppe.
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0

from base import FetchPluginBase

## Share data tracker querying quotes of Raiffeisen fonds
#
#  The used source has some special attributes. The WKN is the austrian
#  one and the date is without year. The date will complete using the
#  current year.
#
class RaiffeisenFetchPlugin(FetchPluginBase):

    _specific_properties   = ['currency.code', 'share.isin', 'share.name', 'net', 'quote.pchange',
                              'quote.price', 'share.wkn', ]

    ## Version number of the interface to access and convert the data
    interface_version = 2

    ## Name of the instance logger
    logger_name = 'smm.quote.raiffeisen'

    ## Name of the data source
    name = u'Raiffeisen Bankengruppe'

    ## Unique provider ID
    provider_id = 300

    ## Decimal separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see tsep
    dsep = u','

    ## Thousands separator for converting strings to decimal values
    #  \see attr2decimal()
    #  \see dsep
    tsep = None

    ## URL to query the data
    provider_url = \
        'http://rdg.ttweb.net/redesign/fundTool/kurzInfo.asp?ID=%(isin)s&output=print'
    
    ## Obtain information from the data provider
    #
    #  \param self     The object instance itself
    #  \param request  See example
    #
    #  \par Example:
    #  \code
    #  >>> import pprint
    #  >>> pprint.pprint(request)
    #  {u'AT0000936513': {'currency.code': u'EUR',
    #                     'share.isin': u'AT0000936513',
    #                     'share.wkn': u'973205'}}
    #  >>> provider = smm.quote.raiffeisen.RaiffeisenFetchPlugin()
    #  >>> pprint.pprint(result)
    #  {u'AT0000936513': {'currency.code': u'EUR',
    #                     'errormsg': None,
    #                     'kind': u'fond',
    #                     'quote.datetime': datetime.datetime(2010, 5, 14, 0, 0),
    #                     'quote.nchange': Decimal("10.59"),
    #                     'quote.pchange': Decimal("0.0427"),
    #                     'quote.price': Decimal("258.67"),
    #                     'share.isin': u'AT0000936513',
    #                     'share.issuer': u'Raiffeisen KAG',
    #                     'share.name': u'Raiffeisen-Osteuropa-Aktien R A',
    #                     'share.wkn': u'93651',
    #                     'source.id': 300,
    #                     'source.name': u'Raiffeisen Bankengruppe',
    #                     'success': True}}
    #  \endcode
    def getCurrent(self, request):
        # unitialise internal variables
        info = {}

        for key in request:
            isin     = request[key].get('share.isin', None)
            currency = request[key].get('currency.code', None)
            if not isin:
                self.logger.error(
                    _(u'Fr diese Abfrage wurde keine ISIN angegeben. Es wird daher keine Abfrage bei Raiffeisen durchgefhrt.'),
                    )
                continue

            # compose URL
            url = self.provider_url % {'isin': isin}

            html = self.getHTML(info, isin, url, 'Fondsgesellschaft')
            if not html:
                continue

            # create a HTML soup
            soup = self.getSoup(html)
            tables = soup('table')
        
            # create shortcuts
            stock       = info[isin]
            quotes      = tables[2]('tr')[3]
            master_data = tables[2]('tr')[5]
            data = {}

            # set static data
            data['kind'] = u'fond'
        
            # extract master data
            data['share.name']    = tables[0].text
            data['share.issuer']  = quotes('td')[0].string.strip()
            data['share.wkn']     = master_data('tr')[2]('td')[1].contents[0].string.strip()
            data['share.isin']    = master_data('tr')[1]('td')[1].contents[0].string.strip()
            data['currency.code'] = master_data('tr')[3]('td')[1].contents[0].string.strip()
        
            # extract quotes
            self.attr2decimal(data, 'quote.price', quotes('td')[2])
            self.attr2decimal(data, 'quote.nchange', quotes('td')[5].contents[0])
            self.attr2decimal(data, 'quote.pchange', quotes('td')[6].contents[0])

            # extract date
            self.timestamp2datetime(
                data,
                quotes('td')[4].string.strip(),
                u'%d.%m.'
                )

            # check currency
            if currency and currency != data['currency.code']:
                continue
              
            # apply data
            stock.update(data)
    
        return info
