#!/usr/bin/env  python
# -*- coding: Latin-1 -*-
#
# SMM Share and Money Management
# Copyright (c) 2007-2009 Carsten Grohmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the
# License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

## \file tests/test_objects.py
#
# \brief Test database related stuff like creating of objects
#
# \author Carsten Grohmann <mail (at) carstengrohmann (dot) de>
#
# \par License:
# GPL version 2.0


# import general python modules
import datetime
import decimal
import logging
import sys
import types
import unittest

# import project modules
from storm.locals import *
import smm

## \brief Test basic functionality with all objects from smm.objects
#
#  Tests creation, searching, adding and removing of all object classes
class TestObjects(unittest.TestCase):

    ## Creates and initialise a (temporary) in-memory database
    def setUp(self):
        smm.database.init_storm('sqlite:')
        smm.db_setup.create_structure()
        smm.db_setup.load_initial_data()
        self.store = smm.database.store

        # log error messages only
        logging.root.level = logging.ERROR

    ## Check equality of two objects
    #
    #  The function compares the \c id attribute as well as identity of
    #  both objects. The identity is getting using \c id().
    #
    #  \param self   The object instance itself
    #  \param first  First object
    #  \param second Second object
    def _checkEquality(self, first, second):
        self.failUnless(
            second.id == first.id,
            'Got %s object with unexpected value in attribute "id"' % \
                first.__class__.__name__
            )
        self.failUnless(
            id(second) == id(first),
            "Got new %s object expected is an existing object" % \
                first.__class__.__name__
            )


    ## Tests creating, searching and destroying smm.objects.Exchange objects
    def test_Exchange_objects(self):
        # create new exchange object
        ex = smm.Exchange(
            symbol=u'XX',
            description=u'Test Exchange'
            )

        # check store functionality
        self.failIf(
            Store.of(ex),
            "It seems that the Exchange object is already stored"
            )
        ex.add2db()
        self.failUnless(ex.id, "Exchange object didn't add to store")
        self.failUnless(
            Store.of(ex) is self.store,
            "Exchange object wasn't added to the store"
            )

        # search exchange
        obj = self.store.get(smm.Exchange, ex.id)
        self.failUnless(obj, "Can't get Exchange (%d, %s, %s)!" % (
            ex.id,
            ex.symbol,
            ex.description
            ))

        # Check for automatic conversion to unicode
        smm.Exchange(symbol='XX', description='TEST')
        smm.Exchange(symbol=u'XX', description=u'TEST')


    ## Test the smm.objects.Exchange.get() command
    def test_Exchange_get(self):
        # test Exchange.get(id=<AAA>)
        #============================
        ex = smm.Exchange.get(id=1)
        self.failUnless(ex, "Exchange object doesn't exists")
        self.failUnless(ex.id, 'Missing primary key for %r' % ex)

        # get non existing object
        ex = smm.Exchange.get(id=sys.maxint)
        self.failIf(ex, 'Got Exchange object that should not exists: %r' % ex)

        # test smm.Exchange.get(symbol=<AAA>)
        #====================================
        ex = smm.Exchange.get(symbol=u'DE')
        self.failUnless(ex.symbol == u'DE', 'Got wrong exchange symbol!')
        self.failUnless(
            ex.description == smm.db_setup.exchange_list[u'DE'],
            'Got wrong exchange description'
            )
        ex = smm.Exchange.get(symbol=u'XTSX')
        self.failIf(ex, 'Got Exchange object that should not exists: %r' % ex)

        # test smm.Exchange.get(city=<AAA>)
        #====================================
        ex = smm.Exchange.get(city=u'Berlin')
        self.failUnless(ex.symbol == u'BE', 'Got wrong exchange symbol!')
        self.failUnless(
            ex.description == smm.db_setup.exchange_list[u'BE'],
            'Got wrong exchange description'
            )

        # check for case in-sensitive search for pure ascii
        ex = smm.Exchange.get(city=u'BERLIN')
        self.failUnless(ex.symbol == u'BE', 'Got wrong exchange symbol!')
        self.failUnless(
            ex.description == smm.db_setup.exchange_list[u'BE'],
            'Got wrong exchange description'
            )

        # check for case in-sensitive search for pure ascii
        ex = smm.Exchange.get(city=u'MNCHEN')
        self.failUnless(ex.symbol == u'MU', 'Got wrong exchange symbol!')
        self.failUnless(
            ex.description == smm.db_setup.exchange_list[u'MU'],
            'Got wrong exchange description'
            )

        # search non-existing exchange
        ex = smm.Exchange.get(city='Dresden')
        self.failIf(ex, 'Got Exchange object that should not exists: %r' % ex)



    ## Tests creating, searching and destroying smm.objects.Share objects
    def test_Share_objects(self):
        # create new Share object
        share = smm.Share(
            name='TestShare',
            symbol=None,
            isin='DE9991002003',
            wkn='100200',
            kind='share',
            )

        # check store functionality
        self.failIf(
            Store.of(share),
            "It seems that the Share object is already stored"
            )
        share.add2db()
        self.failUnless(share.id, "Share object didn't add to store")
        self.failUnless(
            Store.of(share) is self.store,
            "Share object wasn't added to the store"
            )

        # search exchange
        obj = self.store.get(smm.Share, share.id)
        self.failUnless(obj, "Can't get Share before created")


    ## Test the smm.objects.Share.get() command
    def test_Share_get(self):
        # test Share.get(id=<AAA>)
        #=========================
        sh = smm.Share.get(id=1)
        self.failUnless(sh, "Share object doesn't exists")
        self.failUnless(sh.id, 'Missing primary key for %r' % sh)

        # get non existing object
        sh = smm.Share.get(id=sys.maxint)
        self.failIf(sh, 'Got Share object that should not exists: %r' % sh)

        # test smm.Share.get(symbol=<AAA>)
        #=================================
        sh = smm.Share.get(symbol='ADS')
        self.failUnless(
            sh.symbol == 'ADS',
            'Got wrong share symbol (got: "%s", expect: "ADS")!' % sh.symbol
            )
        self.failUnless(
            sh.name == 'adidas AG',
            'Got wrong share name (got: "%s", expect: "adidas AG")!' % sh.name
            )
        sh = smm.Share.get(symbol='XTSX')
        self.failIf(sh, 'Got Share object that should not exists: %r' % sh)

        # test smm.Share.get(isin=<AAA>)
        #===============================
        sh = smm.Share.get(isin='DE0005003404')
        self.failUnless(
            sh.isin == 'DE0005003404',
            'Got share with wrong ISIN (got: "%s", expect: "DE0005003404")!' % sh.isin
            )

        # test smm.Share.get(isin=<AAA>)
        #===============================
        sh = smm.Share.get(wkn='500340')
        self.failUnless(
            sh.wkn == '500340',
            'Got share with wrong WKN (got: "%s", expect: "500340")!' % sh.wkn
            )


    ## Tests creating, searching and destroying smm.objects.Currency objects
    def test_Currency_objects(self):
        # create new Currency object
        currency = smm.Currency(
            code='XTS',
            description=u'Test Currency'
            )

        # check store functionality
        self.failIf(
            Store.of(currency),
            "It seems that the Currency object is already stored"
            )
        currency.add2db()
        self.failUnless(currency.id, "Currency object didn't add to store")
        self.failUnless(
            Store.of(currency) is self.store,
            "Currency object wasn't added to the store"
            )

        # search currency
        obj = self.store.get(smm.Currency, currency.id)
        self.failUnless(obj, "Can't get Currency (%d, %s, %s)!" % (
            currency.id,
            currency.code,
            currency.description
            ))

        # Check for automatic conversion to unicode
        smm.Currency(code='XTS', sign='$', description='TEST')
        smm.Currency(code=u'XTS2', sign=u'\N{DOLLAR SIGN}', description=u'TEST')

    ## Test the smm.objects.Currency.get() command
    def test_Currency_get(self):
        # get an existing Currency object and check if it's already stored
        currency = smm.Currency.get(id=1)
        self.failUnless(
            currency,
            "Currency object doesn't exists"
            )
        self.failUnless(
            currency.id,
            'Missing primary key for %r' % currency
            )

        # get non existing object
        currency = smm.Currency.get(id=sys.maxint)
        self.failIf(currency, 'Got Currency object that should not exists: %r' % currency)

        # get an existing Currency object specified by the currency code
        cur = smm.Currency.get(code='EUR')
        self.failUnless(
            cur.code == 'EUR',
            'Got wrong currency symbol (found: %s, expected: EUR)!' % cur.code
            )

        # Check for non-existing currency
        cur = smm.Currency.get(code='XTSX')
        self.failIf(cur, 'Got Currency object that should not exists: %r' % cur)


    ## Tests creating, searching and destroying smm.objects.Share objects
    def test_DailyQuote_objects(self):
        # create Share object
        share = smm.Share.get(1)
        # create new DailyQuote object
        quote = smm.DailyQuote(
            share=share,
            date=datetime.date.today()
            )

        quote.add2db()
        self.failUnless(quote.id, "DailyQuote object didn't add to store")
        self.failUnless(
            Store.of(quote) is self.store,
            "DailyQuote object wasn't added to the store"
            )

        # search currency
        obj = self.store.get(smm.DailyQuote, quote.id)
        self.failUnless(obj, "Can't get DailyQuote (%d, %s)!" % (
            quote.id,
            quote.date
            ))


    ## Test the smm.objects.DailyQuote.get() command
    def test_DailyQuote_get(self):
        # create helper objects first
        share = smm.Share.get(1)
        date = datetime.date.today()
        dq = smm.DailyQuote(
            date=date,
            share=share,
            open=decimal.Decimal(200),
            close=decimal.Decimal(100)
            )

        # add to store
        dq.add2db()

        # run tests
        dq_new = smm.DailyQuote.get(share=share, date=date)
        self.failUnless(dq_new, "DailyQuote object doesn't exists")
        self._checkEquality(dq, dq_new)

    ## Tests the smm.Objects.DailyQuote.getLast() command
    def test_DailyQuote_getLast(self):
        # create helper objects first
        share = smm.Share.get(1)
        cur1 = smm.Currency.get(code="EUR")
        cur2 = smm.Currency.get(code="USD")
        cur3 = smm.Currency.get(code="CAD")

        # create quotes
        quote1 = smm.DailyQuote(
            close=decimal.Decimal(5),
            currency=cur1,
            date=datetime.date(2007,12,12),
            open=decimal.Decimal(1),
            share=share,
            )
        quote2 = smm.DailyQuote(
            close=decimal.Decimal(5),
            currency=cur1,
            date=datetime.date(2007,12,14),
            open=decimal.Decimal(1),
            share=share,
            )

        # add to store
        quote1.add2db(False)
        quote2.add2db()

        # run test
        last = smm.DailyQuote.getLast(share)
        self.failUnless(
            last == quote2,
            "Doesn't get the newest quote "
            "(got: %r, expect: %r)!" % (last, quote2)
            )
        self.failUnless(
           last.date == datetime.date(2007,12,14),
           'Quote has wrong date '
           '(got: %s, expect: 2007-12-14)!' % last.date
           )

        # add a third quote with a different currency
        quote3 = smm.DailyQuote(
            close=decimal.Decimal(6),
            currency=cur2,
            date=datetime.date(2007,12,16),
            open=decimal.Decimal(4),
            share=share,
            )
        quote3.add2db()

        # rerun test
        last = smm.DailyQuote.getLast(share)
        self.failUnless(
            last == quote3,
            "Doesn't get the newest quote "
            "(got: %r, expect: %r)!" % (last, quote3)
            )
        self.failUnless(
           last.date == datetime.date(2007,12,16),
           'Quote has wrong date '
           '(got: %s, expect: 2007-12-16)!' % last.date
           )

        # do another test specify the currency
        last = smm.DailyQuote.getLast(share, cur1)
        self.failUnless(
            last == quote2,
            "Doesn't get the newest quote for currency %s "
            "(got: %r, expect: %r)!" % (cur1, last, quote2)
            )
        self.failUnless(
           last.date == datetime.date(2007,12,14),
           'Quote has wrong date '
           '(got: %s, expect: 2007-12-14)!' % last.date
           )

        # Query never stored quotes
        last = smm.DailyQuote.getLast(share, cur3)
        self.failUnless(
            last == None,
            'Got never stored quote '
            '(got: %r, expect: None)!' % last
            )




    ## Tests creating, searching and destroying smm.objects.Transaction objects
    def test_Transaction_objects(self):
        # create helper objects first
        sh = smm.Share.get(symbol='ADS')
        cur = smm.Currency.get(code='EUR')
        pos = smm.Position(
            currency=cur,
            share=sh
            )
        pos.add2db()

        # create transaction object
        transaction = pos.buy(
            count=decimal.Decimal(500),
            value=decimal.Decimal('10.4'),
            fee=decimal.Decimal('4.99'),
            date=datetime.date.today(),
            exchange=smm.Exchange.get(symbol='DE')
            )

        # check store functionality
        self.failUnless(transaction.id, "Transaction object didn't add to store")
        self.failUnless(
            Store.of(transaction) is self.store,
            "Transation object wasn't added to the store"
            )


    ## Test the smm.objects.Transaction.get() command
    def test_Transaction_get(self):
        # create helper objects first
        sh = smm.Share.get(symbol='ADS')
        cur = smm.Currency.get(code='EUR')
        pos = smm.Position(
            currency=cur,
            share=sh
            )
        pos.add2db()
        t1 = pos.buy(
            count=decimal.Decimal(58),
            value=decimal.Decimal(16),
            fee=decimal.Decimal(9),
            date=datetime.date(2007,11,5)
        )
        t2 = pos.sell(
            count=decimal.Decimal(-58),
            value=decimal.Decimal(10),
            fee=decimal.Decimal(9),
            date=datetime.date(2007,11,6)
        )
        t3 = pos.buy(
            count=decimal.Decimal(100),
            value=decimal.Decimal(7),
            fee=decimal.Decimal(20),
            date=datetime.date(2007,11,10)
        )

        # test Transaction.get(id=<AAA>)
        #===============================
        t1_new = smm.Transaction.get(id=t1.id)
        self._checkEquality(t1, t1_new)

        # test Transations.get() - fetch all
        #===================================
        # add fourth transaction
        t4 = pos.buy(
            count=decimal.Decimal(500),
            value=decimal.Decimal(7),
            fee=decimal.Decimal(20),
            date=datetime.date(2007,11,20)
        )

        trans_list = smm.Transaction.get()

        # create a set of id numbers of all transactions
        ids_expected = set([t1.id, t2.id, t3.id, t4.id])
        ids_queried = set()
        [ids_queried.add(trans.id) for trans in trans_list]

        self.failUnless(
            ids_expected == ids_queried,
            "Transaction.get() returned unexpected transactions (got: %s, expect: %s)." % (
                ids_queried,
                ids_expected
                )
            )


    ## Test the smm.objects.Transaction.getVolume() command
    def test_Transaction_getVolume(self):
        # create helper objects first
        sh = smm.Share.get(symbol='ADS')
        cur = smm.Currency.get(code='EUR')
        pos = smm.Position(
            currency=cur,
            share=sh
            )
        pos.add2db()

        # buy transaction
        t1 = pos.buy(
            count=decimal.Decimal(50),
            value=decimal.Decimal(20),
            fee=decimal.Decimal(10),
            date=datetime.date(2007,11,1)
            )
        # second buy transaction
        t2 = pos.buy(
            count=decimal.Decimal(10),
            value=decimal.Decimal(10),
            date=datetime.date(2007,11,2)
            )
        # sell transaction
        t3 = pos.sell(
            count=decimal.Decimal(-10),
            value=decimal.Decimal(10),
            date=datetime.date(2007,11,3)
            )
        # sell transaction with fee
        t4 = pos.sell(
            count=decimal.Decimal(-10),
            value=decimal.Decimal(10),
            fee=decimal.Decimal(5),
            date=datetime.date(2007,11,3)
            )
        # divident transaction
        t5 = pos.dividend(
            count=decimal.Decimal(50),
            value=decimal.Decimal('0.20'),
            date=datetime.date(2007,11,20)
            )
        # split transaction
        t6, t7 = pos.split(1, 4, datetime.date(2007,11,25))

        # Test every transaction as a single transaction
        volume = t1.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(1010),
            'Got wrong transaction volume (got: %s, expect: 1010).' % volume
            )
        volume = t2.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(100),
            'Got wrong transaction volume (got: %s, expect: 100).' % volume
            )
        volume = t3.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(-100),
            'Got wrong transaction volume (got: %s, expect: -100).' % volume
            )
        volume = t4.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(-105),
            'Got wrong transaction volume (got: %s, expect: -105).' % volume
            )
        volume = t5.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(10),
            'Got wrong transaction volume (got: %s, expect: 10).' % volume
            )
        volume = t6.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(0),
            'Got wrong transaction volume (got: %s, expect: 0).' % volume
            )
        volume = t7.getVolume()
        self.failUnlessEqual(
            volume,
            decimal.Decimal(0),
            'Got wrong transaction volume (got: %s, expect: 0).' % volume
            )


    ## \brief Tests the automatic conversion of attributes
    #
    #  This functionality is used in all classes derivated from
    #  smm.objects.SMMBaseObject
    def test_AttributeConversion(self):
        # create a Share object and test all conversions individually
        sh = smm.Share()
        result = sh._toUnicode('123')
        self.failUnless(
            result == u'123',
            'Conversion to Unicode failed (got: %s, '
            'expected: 123)!' % result
            )
        self.failUnless(
            type(result) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(result)
            )
        result = sh._toUnicode(123)
        self.failUnless(
            result == u'123',
            'Conversion to Unicode failed (got: "%s", '
            'expected: 123)!' % result
            )
        self.failUnless(
            type(result) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(result)
            )
        result = sh._toUnicode(None)
        self.failUnless(
            result == None,
            'Conversion to Unicode failed (got: "%s", '
            'expected: None)!' % result
            )
        self.failUnless(
            type(result) == types.NoneType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'NoneType'>)!" % type(result)
            )

        result = sh._toDecimal(123)
        self.failUnlessEqual(
            result,
            decimal.Decimal('123'),
            'Conversion to Decimal failed (got: %s, '
            'expected: 123)!' % result
            )
        self.failUnless(
            isinstance(result, decimal.Decimal),
            "Conversion to Decimal failed (got: %s, "
            "expected: <class 'decimal.Decimal'>)!" % type(result)
            )
        result = sh._toDecimal(float('1.2'))
        self.failUnless(
            result == decimal.Decimal('1.2'),
            'Conversion to Decimal failed (got: %s, '
            'expected: 1.2)!' % result
            )
        self.failUnless(
            isinstance(result, decimal.Decimal),
            "Conversion to Decimal failed (got: %s, "
            "expected: <class 'decimal.Decimal'>)!" % type(result)
            )
        result = sh._toDecimal(None)
        self.failUnless(
            result == None,
            'Conversion to Decimal failed (got: "%s", '
            'expected: None)!' % result
            )
        self.failUnless(
            type(result) == types.NoneType,
            "Conversion to Decimal failed (got: %s, "
            "expected: <type 'NoneType'>)!" % type(result)
            )
        result = sh._toDecimal('kA')
        self.failUnless(
            result == None,
            'Conversion to Decimal failed (got: "%s", '
            'expected: None)!' % result
            )
        self.failUnless(
            type(result) == types.NoneType,
            "Conversion to Decimal failed (got: %s, "
            "expected: <type 'NoneType'>)!" % type(result)
            )


    ## Tests creating smm.objects.Watch objects
    def test_Watch_objects(self):
        # zu testen:
        #  - Watch.change

        # create helper objects first
        sh = smm.Share.get(symbol='ADS')

        # create Watch object
        watch = smm.Watch(
            share=sh,
            created=datetime.date(2007, 12, 31),
            expire=datetime.date(2008, 12, 31),
            notes='No comment!'
            )

        # check store functionality
        watch.add2db()
        self.failUnless(watch.id, "Watch object didn't add to store")
        self.failUnless(
            Store.of(watch) is self.store,
            "Share object wasn't added to the store"
            )

        # check relations
        self.failUnless(
            id(sh.watch) == id(watch),
            "Share object doesn't point to the Watch object."
            )
        self.failUnless(
            sh.watch_id == watch.id,
            "Share object has an other id of the watch object that the "
            "watch object itself."
            )


    ## Test the smm.objects.Watch.get() command
    def test_Watch_get(self):
        # create helper objects first
        share_ALV = smm.Share.get(symbol='ALV')
        share_SIE = smm.Share.get(symbol='SIE')
        watch_ALV = smm.Watch(
            share=share_ALV,
            created=datetime.date(2007, 12, 31),
            )
        watch_SIE = smm.Watch(
            share=share_SIE,
            created=datetime.date(2008, 1, 1),
            )
        # add to store
        watch_ALV.add2db(False)
        watch_SIE.add2db()

        # test Watch.get(id=<AAA>)
        #=========================
        w_new = smm.Watch.get(id=watch_ALV.id)
        self._checkEquality(watch_ALV, w_new)

        # test Watch.get(share=<AAA>)
        #============================
        result_set = smm.Watch.get(share=share_SIE)
        self.failUnless(
            result_set.count() == 1,
            'Mismatch in number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )
        w_new = result_set[0]
        self._checkEquality(watch_SIE, w_new)

        # test Watch.get(expired=True)
        #=============================
        result_set = smm.Watch.get(expired=True)
        self.failUnless(
            result_set.count() == 0,
            'Mismatch in number of expected open positions '
            '(found: %d, expected: 0)!' % result_set.count()
            )
        watch_ALV.expire = datetime.date.today() - datetime.timedelta(days=1)
        result_set = smm.Watch.get(expired=True)
        self.failUnless(
            result_set.count() == 1,
            'Mismatch in number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )
        watch_ALV_new = result_set[0]
        self._checkEquality(watch_ALV, watch_ALV_new)

        # Expire the second watch list entry
        watch_SIE.expire = datetime.date.today() - datetime.timedelta(days=1)
        result_set = smm.Watch.get(expired=True)
        self.failUnless(
            result_set.count() == 2,
            'Mismatch in number of expected open positions '
            '(found: %d, expected: 2)!' % result_set.count()
            )

        # create a set of id numbers of all transactions
        ids_expected = set([watch_ALV.id, watch_SIE.id])
        ids_queried = set()
        [ids_queried.add(watch.id) for watch in result_set]

        self.failUnless(
            ids_expected == ids_queried,
            "Watch.get(expired=True) returned unexpected Watch objects (got: %s, expect: %s)." % (
                ids_queried,
                ids_expected
                )
            )

        # test Watch.get(share=<AAA>, expired=True)
        #==========================================
        result_set = smm.Watch.get(share=share_ALV, expired=True)
        self.failUnless(
            result_set.count() == 1,
            'Mismatch in number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )
        watch_ALV_new = result_set[0]
        self._checkEquality(watch_ALV, watch_ALV_new)


    ## \brief Tests setting / update of attributes
    #
    #  This functionality is used in all classes derivated from
    #  smm.objects.SMMBaseObject
    def test_setAttrs(self):
        # create a Currency object and run tests using it
        sh = smm.Currency()
        sh._setAttrs(
            ignoreEmpty=True, code='123', sign=u'BLU', description='text'
            )
        self.failUnless(
            sh.code == u'123',
            "Got wrong 'code' attribute (got: %s, "
            "expect: u'123')!" % sh.code
            )
        self.failUnless(
            type(sh.code) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(sh.code)
            )
        self.failUnless(
            sh.sign == u'BLU',
            "Got wrong 'sign' attribute (got: %s, "
            "expect: u'BLU')!" % sh.sign
            )
        self.failUnless(
            type(sh.sign) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(sh.sign)
            )
        self.failUnless(
            sh.description == u'text',
            "Got wrong 'description' attribute (got: %s, "
            "expect: u'test')!" % sh.description
            )
        self.failUnless(
            type(sh.description) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(sh.description)
            )

        # delete a single attribute
        sh._setAttrs(ignoreEmpty=False, description=None)
        self.failUnless(
            sh.description == None,
            "Got wrong 'description' attribute (got: %s, "
            "expect: None)!" % sh.description
            )
        self.failUnless(
            type(sh.description) == types.NoneType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'NoneType'>)!" % type(sh.description)
            )

        # check ignoring attribute
        sh._setAttrs(ignoreEmpty=False, description="text")
        sh._setAttrs(ignoreEmpty=True, description=None)
        self.failUnless(
            sh.description == u'text',
            "Got wrong 'description' attribute (got: %s, "
            "expect: u'test')!" % sh.description
            )
        self.failUnless(
            type(sh.description) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(sh.description)
            )

        # check conversion of empty unicode attributes
        sh._setAttrs(ignoreEmpty=False, description="text")
        sh._setAttrs(ignoreEmpty=False, description=u'')
        self.failUnless(
            sh.description == None,
            "Got wrong 'description' attribute (got: %s, "
            "expect: None)!" % sh.description
            )
        self.failUnless(
            type(sh.description) == types.NoneType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'NoneType'>)!" % type(sh.description)
            )

        # check cutting off for unicode strings
        sh._setAttrs(ignoreEmpty=False, description="text")
        sh._setAttrs(ignoreEmpty=False, description=u' a\n  ')
        self.failUnless(
            sh.description == u'a',
            "Got wrong 'description' attribute (got: '%s', "
            "expect: ' a')!" % sh.description
            )
        self.failUnless(
            type(sh.description) == types.UnicodeType,
            "Conversion to Unicode failed (got: %s, "
            "expected: <type 'unicode'>)!" % type(sh.description)
            )


    ## Tests creating smm.objects.Position objects
    def test_Position_objects(self):
        # create helper objects first
        cur = smm.Currency.get(code='EUR')
        ex = smm.Exchange.get(symbol='DE')
        sh = smm.Share.get(symbol='ADS')

        # Create first Postion object
        pos = smm.Position(
            currency=cur,
            share=sh,
            notes='First Position :-)'
            )
        pos.add2db()

        # check position details
        self.failUnless(
            pos.id,
            "Position object didn't add to store"
            )
        self.failUnless(
            Store.of(pos) is self.store,
            "Postion object wasn't added to the store"
            )
        self.failUnless(
            pos.currency == cur,
            'Got unexpected position currency '
            '(found: %s, expected: EUR)!' % pos.currency.code
            )
        self.failUnless(
            pos.share == sh,
            'Got unexpected position share '
            '(found: %s, expected: ADS)!' % pos.share.symbol
            )
        self.failUnless(
            pos.notes == u'First Position :-)',
            'Got unexpected position notes '
            '(found: %s, expected: u"First Position :-)")!' % pos.notes
            )
        self.failUnless(
            pos.open,
            'Got unexpected position state '
            '(found: %s, expected: True)!' % pos.open
            )
        self.failUnless(
            pos.transactions.count() == 0,
            'Got unexpected number of position transactions '
            '(found: %d, expected: 0)!' % pos.transactions.count()
            )

        # check transactions
        t1 = pos.buy(
            count=decimal.Decimal(500),
            value=decimal.Decimal('10.4'),
            fee=decimal.Decimal('4.99'),
            date=datetime.date.today(),
            exchange=ex,
            )
        self.failUnless(
            pos.transactions.count() == 1,
            'Got unexpected number of position transactions '
            '(found: %d, expected: 1)!' % pos.transactions.count()
            )
        ids_expected = set([t1.id])
        ids_queried = set()
        [ids_queried.add(trans.id) for trans in pos.transactions]
        self.failUnless(
            t1 in pos.transactions,
            'Got unexpected position transactions '
            '(found: %s, expected: %s)!' % (
                ids_queried,
                ids_expected
                )
            )

         # clean positions
        result_set = self.store.find(smm.Position)
        result_set.remove()
        self.store.commit()
        self.store.flush()


    ## Test the smm.objects.Position.get() command
    def test_Position_get(self):
        # create helper objects first
        cur = smm.Currency.get(code='EUR')
        sh = smm.Share.get(symbol='ADS')
        pos = smm.Position(
            currency=cur,
            share=sh
            )
        pos.add2db()

        # test Position.get(id=<AAA>)
        #============================
        pos_new = smm.Position.get(id=pos.id)
        self.failUnless(pos_new, "Position object doesn't exists")
        self.failUnless(pos_new.id, 'Missing primary key for %r' % pos_new)

        # get non existing object
        pos_new = smm.Position.get(id=sys.maxint)
        self.failIf(
            pos_new,
            'Got Position object that should not exists: %r' % pos_new
            )

        # test Position.get(open=True)
        #=============================
        result_set = smm.Position.get(open=True)
        self.failUnless(
            result_set.count() == 1,
            'Unexpected number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )

        # test Position.get(open=False)
        #=============================
        pos.open = False
        result_set = smm.Position.get(open=False)
        self.failUnless(
            result_set.count() == 1,
            'Unexpected number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )

        # test Position.get(share=<AAA>)
        #===============================
        result_set = smm.Position.get(share=sh)
        self.failUnless(
            result_set.count() == 1,
            'Unexpected number of expected open positions '
            '(found: %d, expected: 1)!' % result_set.count()
            )
        pos = result_set[0]
        self.failUnless(
            pos.share == sh,
            'Got unexpected Position object for wrong share '
            '(got: %s, expect: ALV)!' % pos.share.symbol
            )

        sh = smm.Share.get(symbol='ALV')
        result_set = smm.Position.get(share=sh)
        self.failUnless(
            result_set.count() == 0,
            'Got Position object that should not exists'
            )

    ## Tests the smm.objects.Position.getPerf() command.
    #
    #  Additionally will be tested:
    #   - smm.objects.Position.getShareCount()
    def test_Position_getPerf(self):
        # create helper objects first
        cur = smm.Currency.get(code='EUR')
        ex = smm.Exchange.get(symbol='DE')
        sh = smm.Share(
            symbol='XX_CPTEST',
            kind='share',
            name='Test share to test Position.getPerf()',
            exchange=ex
            )
        pos = smm.Position(
            currency=cur,
            share=sh
            )
        pos.add2db()
        pos.buy(
            count=decimal.Decimal(50),
            value=decimal.Decimal(20),
            fee=decimal.Decimal(10),
            date=datetime.date(2007,11,1),
            exchange=ex,
            )

        # Check share count
        count = pos.getShareCount()
        self.failUnless(
            count == 50,
            'Got wrong share count for position '
            '(got: %d, expect: 50)!' % count
            )

        # The performance without fees should be abs: 0 Eur, rel 0%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal(0),
            'Got absolute performance (got: %s, expect: 0).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(0),
            'Got wrong relative performance (got: %s, expect: 0).' % perf_rel
            )

        # The performance with fees should be abs: -10 Eur, rel -0.990099%
        (perf_abs, perf_rel) = pos.getPerf(True)
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('-10'),
            'Got absolute performance (got: %s, expect: -10).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal('-1000') / decimal.Decimal('1010'),
            'Got wrong relative performance (got: %s, expect: -0.990099).' % perf_rel
            )

        # add dividend
        pos.dividend(
            count=decimal.Decimal(50),
            value=decimal.Decimal(1),
            date=datetime.date(2007,11,2),
            )

        # The performance should be abs: 50 Eur, rel 5%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal(50),
            'Got wrong absolute performance (got: %s, expect: 50).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(5),
            'Got wrong relative performance (got: %s, expect: 5).' % perf_rel
            )

        # The performance with fees should be abs: 40 Eur, rel 3.960396%
        (perf_abs, perf_rel) = pos.getPerf(True)
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal(40),
            'Got wrong relative performance (got: %s, expect: 40).' % perf_rel
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(4000) / decimal.Decimal(1010),
            'Got wrong absolute performance (got: %s, expect: 3.960396).' % perf_abs
            )

        # add a quote to run a second test
        q1 = smm.DailyQuote(
            date=datetime.date(2007,11,3),
            share=sh,
            currency=cur,
            open=decimal.Decimal(22),
            close=decimal.Decimal(24)
            )
        q1.add2db()

        # The performance should be abs: 250 Eur, rel 25%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('250'),
            'Got wrong absolute performance (got: %s, expect: 250).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(25),
            'Got wrong relative performance (got: %s, expect: 25).' % perf_rel
            )

        # add second "buy"
        pos.buy(
            count=decimal.Decimal(50),
            value=decimal.Decimal(30),
            fee=decimal.Decimal(10),
            date=datetime.date(2007,11,10),
            exchange=ex,
            )

        # Check share count
        count = pos.getShareCount()
        self.failUnless(
            count == 100,
            'Got wrong share count for position '
            '(got: %d, expect: 100)!' % count
            )

        # The performance without fee should be abs: 550 Eur, rel 22%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('550'),
            'Got wrong absolute performance (got: %s, expect: 550).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(22),
            'Got wrong relative performance (got: %s, expect: 22).' % perf_rel
            )
        # The performance with fee should be abs: 530 Eur, rel 21.03174%
        (perf_abs, perf_rel) = pos.getPerf(True)
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('530'),
            'Got wrong absolute performance (got: %s, expect: 530).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(53000)/decimal.Decimal(2520),
            'Got wrong relative performance (got: %s, expect: 21.03174).' % perf_rel
            )

        # add split transaction
        q2 = smm.DailyQuote(
            date=datetime.date(2007,11,25),
            share=sh,
            currency=cur,
            open=decimal.Decimal(9),
            close=decimal.Decimal(10),
            )
        q2.add2db()
        pos.split(1, 3, datetime.date(2007,11,25))

        # Check share count
        count = pos.getShareCount()
        self.failUnless(
            count == 300,
            'Got wrong share count for position '
            '(got: %d, expect: 300)!' % count
            )

        # The performance should be abs: 550 Eur, rel 22%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('550'),
            'Got wrong absolute performance (got: %s, expect: 550).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(22),
            'Got wrong relative performance (got: %s, expect: 22).' % perf_rel
            )
        # The performance with fee should be abs: 530 Eur, rel 21.03174%
        (perf_abs, perf_rel) = pos.getPerf(True)
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('530'),
            'Got wrong absolute performance (got: %s, expect: 530).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(53000)/decimal.Decimal(2520),
            'Got wrong relative performance (got: %s, expect: 21.03174).' % perf_rel
            )

        # add sell transaction
        pos.sell(
            count=decimal.Decimal(-100),
            value=decimal.Decimal(12),
            fee=decimal.Decimal(10),
            date=datetime.date(2007,11,30),
            exchange=ex,
            )

        # Check share count
        count = pos.getShareCount()
        self.failUnless(
            count == 200,
            'Got wrong share count for position '
            '(got: %d, expect: 200)!' % count
            )

        # The performance should be abs: 1150 Eur, rel 46%
        (perf_abs, perf_rel) = pos.getPerf()
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('1150'),
            'Got wrong absolute performance (got: %s, expect: 1150).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(46),
            'Got wrong relative performance (got: %s, expect: 46).' % perf_rel
            )
        # The performance with fee should be abs: 1120 Eur, rel 44.2688%
        (perf_abs, perf_rel) = pos.getPerf(True)
        self.failUnlessEqual(
            perf_abs,
            decimal.Decimal('1120'),
            'Got wrong absolute performance (got: %s, expect: 1120).' % perf_abs
            )
        self.failUnlessEqual(
            perf_rel,
            decimal.Decimal(112000)/decimal.Decimal(2530),
            'Got wrong relative performance (got: %s, expect: 44.2688).' % perf_rel
            )


    ## Tests the smm.Position.getLastQuote() command
    def test_Position_getLastQuote(self):
        # create helper objects first
        cur = smm.Currency.get(code='EUR')
        ex = smm.Exchange.get(symbol='DE')
        sh = smm.Share(
            symbol='XX_GLQ',
            kind='share',
            name='Test share to test getLastQuote()',
            exchange=ex
            )
        pos = smm.Position(
            currency=cur,
            share=sh,
            notes='First Position :-)'
            )
        t1 = pos.buy(
            count=decimal.Decimal(50),
            value=decimal.Decimal(20),
            fee=decimal.Decimal("2.5"),
            date=datetime.date(2007,11,1),
            exchange=ex,
            )
        pos.add2db(False)
        sh.add2db()

        # run first test
        #===============
        (quotation, date) = pos.getLastQuote()
        self.failUnless(
            quotation == t1.value,
            'Got wrong quotation (got: "%s", expect: "%s").' % \
                (quotation, t1.value)
            )
        self.failUnless(
            date == t1.date,
            'Got wrong date (got: "%s", expect: "%s").' % \
                (date, t1.date)
            )
        last = smm.DailyQuote.getLast(sh)
        self.failIf(
            last,
            'Got a daily quote value but excepted "None"'
            )

        # add daily quote and rerun test
        #===============================
        q1 = smm.DailyQuote(
            share=sh,
            open=decimal.Decimal(1),
            close=decimal.Decimal(5),
            currency=cur,
            date=datetime.date(2007,12,12)
            )
        q1.add2db()

        (quotation, date) = pos.getLastQuote()
        self.failUnless(
            quotation == q1.close,
            'Got wrong quotation (got: "%s", expect: "%s").' % \
                (quotation, q1.close)
            )
        self.failUnless(
            date == q1.date,
            'Got wrong date (got: "%s", expect: "%s").' % \
                (date, q1.date)
            )
        last = smm.DailyQuote.getLast(sh)
        self.failUnless(
            last,
            'Got no daily quote value but excepted one'
            )

        # add second transaction and rerun test
        #======================================
        t2 = pos.buy(
            count=decimal.Decimal(50),
            value=decimal.Decimal(20),
            fee=decimal.Decimal("2.5"),
            date=datetime.date(2007,12,20),
            exchange=ex,
            )

        (quotation, date) = pos.getLastQuote()
        self.failUnless(
            quotation == t2.value,
            'Got wrong quotation (got: "%s", expect: "%s").' % \
                (quotation, t2.value)
            )
        self.failUnless(
            date == t2.date,
            'Got wrong date (got: "%s", expect: "%s").' % \
                (date, t2.date)
            )
        last = smm.DailyQuote.getLast(sh)
        self.failUnless(
            last,
            'Got no daily quote value but excepted one'
            )

        # add dividend transaction and rerun test
        #========================================
        pos.dividend(
            count=decimal.Decimal(50),
            value=decimal.Decimal(2),
            date=datetime.date(2007,12,24),
            )

        (quotation, date) = pos.getLastQuote()
        self.failUnless(
            quotation == t2.value,
            'Got wrong quotation (got: "%s", expect: "%s").' % \
                (quotation, t2.value)
            )
        self.failUnless(
            date == t2.date,
            'Got wrong date (got: "%s", expect: "%s").' % \
                (date, t2.date)
            )
        last = smm.DailyQuote.getLast(sh)
        self.failUnless(
            last,
            'Got no daily quote value but excepted one'
            )

if __name__ == '__main__':
    unittest.main()
